#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'

import os
from collections import Iterable
from subprocess import PIPE, Popen
from RouToolPa.Tools.Abstract import Tool



class CGAS(Tool):
    def __init__(self, path="", max_threads=4):

        Tool.__init__(self, "", path=path, max_threads=max_threads)

    @staticmethod
    def _check_input(input_files):

        if not isinstance(input_files, Iterable):
            raise ValueError("%s is not iterable" % input_files)
        return input_files if isinstance(input_files, str) else " ".join(input_files)

    def cat(self, input_files, output=None, capture_output=False):

        options = " %s" % self._check_input(input_files)
        options += " > %s" % output if output else ""

        return self.execute(options, cmd="cat", capture_output=capture_output)

    @staticmethod
    def _parse_grep_options(pattern, count_mode=False, use_regexp=False, num_lines_after_match=None,
                            num_lines_before_match=None, invert_match=False, ignore_case=False, whole_word_match=False,
                            max_matches=None, show_line_numbers=False, recursive_search=False):

        options = " -P" if use_regexp else ""
        options += " -c" if count_mode else ""
        options += " -A %i" % num_lines_after_match if num_lines_after_match else ""
        options += " -B %i" % num_lines_before_match if num_lines_before_match else ""
        options += " -v" if invert_match else ""
        options += " -i" if ignore_case else ""
        options += " -w" if whole_word_match else ""
        options += " -m %i" % max_matches if max_matches else ""
        options += " -n" if show_line_numbers else ""
        options += " -r" if recursive_search else ""
        options += " %s" % pattern

        return options

    def grep(self, pattern, input_files, output=None, count_mode=False, use_regexp=False, num_lines_after_match=None,
             num_lines_before_match=None, invert_match=False, ignore_case=False, whole_word_match=False,
             max_matches=None, show_line_numbers=False, recursive_search=False, capture_output=False):

        options = self._parse_grep_options(pattern, count_mode=count_mode, use_regexp=use_regexp,
                                           num_lines_after_match=num_lines_after_match,
                                           num_lines_before_match=num_lines_before_match, invert_match=invert_match,
                                           ignore_case=ignore_case, whole_word_match=whole_word_match,
                                           max_matches=max_matches, show_line_numbers=show_line_numbers,
                                           recursive_search=recursive_search)
        options += " %s" % self._check_input(input_files)
        options += " > %s" % output if output else ""

        return self.execute(options, cmd="grep", capture_output=capture_output)

    @staticmethod
    def _parse_awk_options(awk_code, separator=None):

        options = " -F'%s'" % separator if separator else ""
        options += " '%s'" % awk_code

        return options

    def awk(self, awk_code, input_files, output=None, separator=None, capture_output=False):

        options = self._parse_awk_options(awk_code, separator=separator)
        options += " %s" % self._check_input(input_files)
        options += " > %s" % output if output else ""

        return self.execute(options, cmd="awk", capture_output=capture_output)

    @staticmethod
    def _parse_sed_options(sed_string, use_regexp=False, disable_print_of_each_line=False):

        options = " -r" if use_regexp else ""
        options += " -n" if disable_print_of_each_line else ""
        options += " '%s'" % sed_string

        return options

    def sed(self, sed_string, input_files, output=None, use_regexp=False, disable_print_of_each_line=False,
            capture_output=False):

        options = self._parse_sed_options(sed_string, use_regexp=use_regexp,
                                          disable_print_of_each_line=disable_print_of_each_line)

        options += " %s" % self._check_input(input_files)
        options += " > %s" % output if output else ""

        return self.execute(options, cmd="sed", capture_output=capture_output)

    def cgas(self, input_files, grep_pattern=None, awk_code=None, sed_string=None, output=None, count_mode=False,
             grep_use_regexp=False, num_lines_after_match=None, num_lines_before_match=None, invert_match=False,
             ignore_case=False, whole_word_match=False, max_matches=None, show_line_numbers=False,
             recursive_search=False, separator=None, sed_use_regexp=False, disable_print_of_each_line=False,
             capture_output=False):

        options = self._check_input(input_files)

        if grep_pattern:
            options += " | grep %s" % self._parse_grep_options(grep_pattern, count_mode=count_mode,
                                                               use_regexp=grep_use_regexp,
                                                               num_lines_after_match=num_lines_after_match,
                                                               num_lines_before_match=num_lines_before_match,
                                                               invert_match=invert_match,
                                                               ignore_case=ignore_case,
                                                               whole_word_match=whole_word_match,
                                                               max_matches=max_matches,
                                                               show_line_numbers=show_line_numbers,
                                                               recursive_search=recursive_search)
        if awk_code:
            options += " | awk %s" % self._parse_awk_options(awk_code, separator=separator)

        if sed_string:
            options += " | sed %s" % self._parse_sed_options(sed_string, use_regexp=sed_use_regexp,
                                                             disable_print_of_each_line=disable_print_of_each_line)

        options += " "

        options += " > %s" % output if output else ""
        return self.execute(options, cmd="cat", capture_output=capture_output)


if __name__ == "__main__":
    pass


#!/usr/bin/env python
from Bio import AlignIO
from Bio.Seq import Seq
from Bio.SeqRecord import SeqRecord


def parse_metamiga_fasta(input_file):
    print("Parsing %s" % input_file)
    seq_dict = {}
    fd = open(input_file, "r")
    sequence = ""
    for line in fd:
        if line[0] == ">":
            if sequence != "":
                seq_dict[seq_id] = SeqRecord(Seq(sequence), id=seq_id, description=gene, name=gene)
                seq_dict[seq_id].annotations["organism"] = species
            annotations_list = line[1:].strip().split("-")
            seq_id = annotations_list[0]
            species = annotations_list[1]
            gene = annotations_list[2]
            sequence = ""
        else:
            sequence += line.strip()
    seq_dict[seq_id] = SeqRecord(Seq(sequence), id=seq_id, description=gene, name=gene)
    seq_dict[seq_id].annotations["organism"] = species
    #print()
    fd.close()
    print("Totaly %i sequences were parsed" % len(seq_dict))
    #for seq in seq_dict:
    #    print(seq, seq_dict[seq])
    return seq_dict


def parse_alignment(input_file, filetype="fasta"):
    print("Parsing alignment file %s" % input_file)
    #print(input_file)
    alignment = AlignIO.read(input_file, filetype)
    #print(alignment)
    print("Totaly %i sequences in alignment" % len(alignment))
    return alignment


def parse_sv(input_file, separator="\t"):
    record_list = []
    with open(input_file, "r") as sv_fd:
        header = sv_fd.readline().strip()
        if header[0] == "#":
            header = header[1:]
        header = header.split("\t")
        for line in sv_fd:
            if line == "\n":
                break
            record_list.append(line.strip().split(separator))
    return header, record_list

"""
def parse_gene_list_file(input_file):
    #TODO: rewrite as class ?
    header, record_list = parse_sv()
"""


#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'

from RouToolPa.Tools.Abstract import Tool


class VEP(Tool):
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "vep", path=path, max_threads=max_threads)

    def parse_options(self):
        """
        if kingdom not in self.kingdoms:
            raise ValueError("Wrong kingdom of life. Allowed: euk, bac, mito, arc")

        options = " --threads %i" % self.threads
        options += " --kingdom %s" % kingdom
        options += " --lencutoff %f" % length_cutoff if length_cutoff else ""
        options += " --reject %f" % reject_cutoff if reject_cutoff else ""
        options += " --evalue %f" % evalue_cutoff if evalue_cutoff else ""
        options += " %s" % input_fasta
        options += " > %s " % gff_file
        options += " 2> %s" % log_file

        return options
        """
        pass

    def prepare_gff(self, input_gff_list, output_prefix):

        sorted_gff = "%s.sorted.gff" % output_prefix
        bgzip_compressed_gff = "%s.sorted.gff.gz" % output_prefix
        sorting_string = "cat %s | grep -v '#' | sort -k1,1 -k4,4n -k5,5n | tee %s | bgzip -c > %s" % (input_gff_list if isinstance(input_gff_list, str) else " ".join(input_gff_list),
                                                                                                                sorted_gff,
                                                                                                                bgzip_compressed_gff)
        indexing_string = "tabix -p gff %s" % bgzip_compressed_gff
        self.execute(cmd=sorting_string)
        self.execute(cmd=indexing_string)

if __name__ == "__main__":
    pass



#!/usr/bin/env python

from collections import OrderedDict

from Bio import SeqIO, Entrez
from RouToolPa.Routines.Sequence import SequenceRoutines
from RouToolPa.Collections.General import IdList, SynDict, TwoLvlDict, IdSet


class EnsemblRoutines(SequenceRoutines):
    def __init__(self):
        SequenceRoutines.__init__(self)

    @staticmethod
    def convert_biomart_protein_annotation_to_gff(biomart_annotations, output_gff, separator="\t",
                                                  extraction_mode="pfam"):

        in_fd = open(biomart_annotations, "r")
        protein_id_header = "Protein ID"

        if extraction_mode == "pfam":
            domain_id_header = "Pfam ID"
            domain_start_header = "Pfam start"
            domain_end_header = "Pfam end"

        header_list = in_fd.readline().strip().split(separator)

        for header_element in  protein_id_header, domain_id_header, domain_start_header, domain_end_header:
            if header_element not in header_list:
                raise ValueError("'%s' is absent in header of the file" % header_element)

        protein_id_index = header_list.index(protein_id_header)
        domain_id_index = header_list.index(domain_id_header)
        domain_start_index = header_list.index(domain_start_header)
        domain_end_index = header_list.index(domain_end_header)

        line_number = 1
        with open(output_gff, "w") as out_fd:
            for line in in_fd:
                line_number += 1
                tmp_list = line.strip("\n").split(separator)
                protein_id = tmp_list[protein_id_index]
                domain_id = tmp_list[domain_id_index]
                domain_start = tmp_list[domain_start_index]
                domain_end = tmp_list[domain_end_index]

                if protein_id == "":
                    print("WARNING!!! Line %i: malformed, no protein id. Skipping..." % line_number)
                    continue
                elif domain_id == "":
                    print("WARNING!!! Line %i: no domain for %s. Skipping..." % (line_number, protein_id))
                    continue
                elif domain_start == "":
                    print("WARNING!!! Line %i: no start coordinate of domain %s for protein %s. Skipping..." % (line_number, domain_id, protein_id))
                    continue
                elif domain_end == "":
                    print("WARNING!!! Line %i: no end coordinate of domain %s for protein %s. Skipping..." % (line_number, domain_id, protein_id))
                    continue

                out_fd.write("%s\tensembl\tdomain\t%s\t%s\t.\t+\t.\tID=%s; Description=%s\n" % (protein_id,
                                                                                                domain_start,
                                                                                                domain_end,
                                                                                                domain_id,
                                                                                                domain_id))
        in_fd.close()

    @staticmethod
    def get_gene_transcript_protein_from_ensembl_pep_fasta(ensembl_fasta, output_file):

        with open(ensembl_fasta, "r") as in_fd:
            with open(output_file, "w") as out_fd:
                for line in in_fd:
                    if line[0] != ">":
                        continue

                    line_list = line.strip().split()
                    protein_id = line_list[0][1:]

                    for entry in line_list:
                        if "gene:" in entry:
                            gene_id = entry.split(":")[1]
                        elif "transcript:" in entry:
                            transcript_id = entry.split(":")[1]

                    out_fd.write("%s\t%s\t%s\n" % (gene_id, transcript_id, protein_id))

    @staticmethod
    def get_longest_pep_per_gene_from_ensembl_pep_dict(protein_dict, output_prefix=None):
        length_file = "%s.protein_length.tsv" % output_prefix
        if output_prefix:
            longest_protein_id_file = "%s.longest_pep.ids" % output_prefix

            len_fd = open(length_file, 'w')
            len_fd.write("#gene_id\tprotein_id\tprotein_length\n")

        data_dict = OrderedDict()
        for protein_id in protein_dict:
            length = len(protein_dict[protein_id].seq)
            description_list = protein_dict[protein_id].description.split()
            #print protein_dict[protein_id]
            #print ''
            #print description_list

            for entry in description_list:
                if "gene:" in entry:
                    gene_id = entry.split(":")[1]
            if output_prefix:
                len_fd.write("%s\t%s\t%i\n" % (gene_id, protein_id, length))
            if gene_id not in data_dict:
                data_dict[gene_id] = protein_id
            else:
                if length > len(protein_dict[data_dict[gene_id]].seq):
                    data_dict[gene_id] = protein_id

        longest_pep_ids = IdList(data_dict.values())
        if output_prefix:
            longest_pep_ids.write(longest_protein_id_file)
            len_fd.close()
        return longest_pep_ids

    def get_longest_pep_per_gene_from_ensembl_pep_file(self, protein_file, output_prefix):
        protein_dict = self.parse_seq_file(protein_file, "parse")
        longest_pep_ids = self. get_longest_pep_per_gene_from_ensembl_pep_dict(protein_dict,
                                                                               output_prefix=output_prefix)

        longest_protein_pep_file = "%s.longest_pep.pep" % output_prefix
        SeqIO.write(self.record_by_id_generator(protein_dict, longest_pep_ids, verbose=True),
                    longest_protein_pep_file,
                    format='fasta')

        return longest_pep_ids




#!/usr/bin/env python
import os
from RouToolPa.Tools.Abstract import Tool



class Bowtie2(Tool):
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "bowtie2", path=path, max_threads=max_threads)

    def index(self, reference, index_name):
        options = "%s %s" % (reference, index_name)
        self.execute(options, cmd="bowtie2-build")

    def align(self,
              bowtie2_index,
              forward_reads_list=None,
              reverse_reads_list=None,
              unpaired_reads_list=None,
              quality_score="phred33",
              alignment_mode="very-sensitive",
              find_discordant_alignments=True,
              find_separated_alignments=True,
              concordant_upper_threshold=None,
              output_prefix="alignment",
              output_format="bam",
              read_group_name="reads",
              PU="x",
              SM="sample",
              platform="Illumina",
              LB="x",
              sort_by_coordinate=True,
              sort_by_name=False,
              max_per_sorting_thread_memory=None):

        options = " -p %i" % self.threads
        options += " --%s" % alignment_mode
        options += " --%s" % quality_score
        options += " -x %s" % bowtie2_index
        options += " -X %i" % concordant_upper_threshold if concordant_upper_threshold else ""
        options += " --no-discordant" if not find_discordant_alignments else ""
        options += " --no-mixed" if not find_separated_alignments else ""
        options += " -1 %s -2 %s" % (",".join(forward_reads_list), ",".join(reverse_reads_list)) \
            if forward_right_reads_list and reverse_left_reads_list else ""
        options += " -U %s" % ",".join(unpaired_reads_list) if unpaired_reads_list else ""

        if sort_by_coordinate or sort_by_name:
            if sort_by_coordinate and sort_by_name:
                raise ValueError("Sorting by both coordinate and read name was requested")
            options += " samtools sort"
            if sort_by_name:
                options += " -n"
            options += " -@ %i" % self.threads
            options += " -m %s" % max_per_sorting_thread_memory if max_per_sorting_thread_memory else self.max_per_thread_memory
            options += " -O %s" % output_format.upper()

        options += " > %s.%s" % (output_prefix, output_format)

        self.execute(options)




#!/usr/bin/env python
import os
import shutil
from RouToolPa.Tools.Abstract import Tool



class GEM(Tool):

    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "gem", path=path, max_threads=max_threads)

    def parse_options(self, reference=None, threads=None, data_type=None, output_prefix=None,
                      kmer_length=None, gem_index=None):

        options = " -T %i" % (threads if threads else self.threads)
        options += " -c %s" % data_type if data_type else ""
        options += " -i %s" % reference if reference else ""
        options += " -o %s" % output_prefix if output_prefix else ""
        options += " -l %i" % kmer_length if kmer_length else ""
        options += " -I %s" % gem_index if gem_index else ""

        return options

    def index(self, reference, output_prefix, threads=None, data_type="dna"):

        options = self.parse_options(reference=reference,
                                     threads=threads,
                                     data_type=data_type,
                                     output_prefix=output_prefix)

        self.execute(options=options, cmd="gem-indexer")

    def create_map_of_mappability(self, kmer_list, output_prefix, gem_index=None, reference=None, threads=None,
                                  convert_to_wig=True):
        if (gem_index is None) and (reference is None):
            raise ValueError("ERROR!!! Neither Gem index nor reference was set!")

        if gem_index is None:
            self.index(reference, output_prefix, threads, data_type='dna')

        gem_idx = gem_index if gem_index else "%s.gem" % output_prefix

        for kmer_len in [kmer_list] if isinstance(kmer_list, int) else kmer_list:
            kmer_mappability_map_prefix = "%s.k%i" % (output_prefix, kmer_len)
            options = self.parse_options(threads=threads, output_prefix=kmer_mappability_map_prefix,
                                         gem_index=gem_idx, kmer_length=kmer_len)

            self.execute(options=options, cmd="gem-mappability")
            if convert_to_wig:
                self.convert_mappability_map_to_wig(map_of_mappability="%s.mappability" % kmer_mappability_map_prefix,
                                                    output_prefix=kmer_mappability_map_prefix,
                                                    gem_index=gem_idx)

    def convert_mappability_map_to_wig(self, map_of_mappability, output_prefix, threads=None, gem_index=None, reference=None):

        if (gem_index is None) and (reference is None):
            raise ValueError("ERROR!!! Neither Gem index nor reference was set!")

        if gem_index is None:
            self.index(reference, output_prefix, threads, data_type='dna')

        gem_idx = gem_index if gem_index else "%s.gem" % output_prefix
        options = self.parse_options(gem_index=gem_idx, output_prefix=output_prefix)
        options += " -i %s" % map_of_mappability

        self.execute(options=options, cmd="gem-2-wig")




#!/usr/bin/python2
import os
import re
from collections import OrderedDict
from math import sqrt

import numpy as np
from scipy.spatial.distance import pdist
from scipy.cluster.hierarchy import linkage, dendrogram, inconsistent, cophenet, fcluster

#import matplotlib
#matplotlib.use('Agg')
import matplotlib.pyplot as plt
plt.ioff()

from RouToolPa.Parsers.Abstract import Metadata, Header


class RecordPWC:
    def __init__(self, gene, basemean, log2foldchange, lfcSE, stat, pvalue, padj,):
        self.gene = gene                              # str
        self.basemean = basemean                      # float
        self.log2foldchange = log2foldchange          # float
        self.lfcSE = lfcSE                            # float
        self.stat = stat                              # float
        self.pvalue = pvalue                          # float
        self.padj = padj                              # float


class MetadataPWC(OrderedDict, Metadata):
    def __init__(self):
        OrderedDict.__init__(self)
        Metadata.__init__(self)


class HeaderPWC(list, Header):

    def __str__(self):
        return "\t".join(self)


class CollectionPWC(OrderedDict):
    def __init__(self, metadata=None, record_dict=None, header=None, pwc_file=None, from_file=True):
        OrderedDict.__init__(self)
        if from_file:
            self.metadata = Metadata()
            with open(pwc_file, "r") as fd:
                self.header = HeaderPWC(fd.readline().strip().split())
                for line in fd:
                    record = self.add_record(line)
                    self[record.gene] = record
        else:
            self.metadata = metadata
            self.header = header
            for gene in record_dict:
                self[gene] = record_dict[gene]

    @staticmethod
    def add_record(line):
        line_list = line.strip().split("\t")
        return RecordPWC(line_list[0].strip("\""), float(line_list[1]), float(line_list[2]), float(line_list[3]),
                         float(line_list[4]), float(line_list[5]), float(line_list[6]))




#!/usr/bin/env python

__author__ = 'mahajrod'
import shutil
from collections import OrderedDict
import numpy as np
from RouToolPa.Tools.Abstract import JavaTool
from RouToolPa.Collections.General import IdList



class CatVariants(JavaTool):
    def __init__(self,  java_path="", max_threads=4, jar_path="", max_memory=None, timelog=None):
        JavaTool.__init__(self, "GenomeAnalysisTK.jar org.broadinstitute.gatk.tools.CatVariants", java_path=java_path,
                          max_threads=max_threads, jar_path=jar_path, max_memory=max_memory,
                          timelog=timelog)

    def parse_options(self, reference, gvcf_list, output, input_is_sorted=False, extension_list=["g.vcf",]):

        #options = " -nt %i" % self.threads # bugs in tool - fails in multithreading mode
        options = " -R %s" % reference

        gvcf_file_list = self.make_list_of_path_to_files_by_extension(gvcf_list, extension_list=extension_list,
                                                                      recursive=False, return_absolute_paths=True)

        for gvcf in gvcf_file_list:
            options += " -V %s" % gvcf

        options += " -out %s" % output
        options += " --assumeSorted" if input_is_sorted else ""

        return options

    def combine_gvcf(self, reference, gvcf_list, output, input_is_sorted=False, extension_list=["g.vcf",],
                     tmp_dir="./tmp_combine_gvcf/", max_files_per_merging=50, iteration=0, threads=None,
                     remove_intermediate_files=False):
        """
        java -jar GenomeAnalysisTK.jar \
           -T GenotypeGVCFs \
           -R reference.fasta \
           --variant sample1.g.vcf \
           --variant sample2.g.vcf \
           -o output.vcf
        """

        filtered_gvcf_list = []
        for filename in gvcf_list:
            for extension in extension_list:
                if extension == filename[-len(extension):]:
                    filtered_gvcf_list.append(filename)
                    break
                
        if len(filtered_gvcf_list) <= max_files_per_merging:
            options = self.parse_options(reference, filtered_gvcf_list, output, input_is_sorted, extension_list=extension_list)
            self.execute(options, runtype="cp")
            if remove_intermediate_files:
                shutil.rmtree(tmp_dir, ignore_errors=True)

        else:
            self.safe_mkdir(tmp_dir)
            iteration_dir = "%s/iteration_%i/" % (tmp_dir, iteration)
            self.safe_mkdir(iteration_dir)

            number_of_files = len(filtered_gvcf_list)

            bins = np.arange(0, number_of_files, max_files_per_merging)
            #print(bins)
            if bins[-1] != number_of_files:
                if number_of_files - bins[-1] < 2:
                    bins[-1] = number_of_files
                else:
                    bins = np.append(bins, number_of_files)

            output_file_list = []
            options_list = []

            merged_files = 0
            for i in range(0, len(bins)-1):
                output_file = "%s/%i.g.vcf" % (iteration_dir, i)
                output_file_list.append(output_file)
                #print(bins[i], bins[i+1])

                merged_files += bins[i+1] - bins[i]
                options_list.append(self.parse_options(reference,
                                                       filtered_gvcf_list[bins[i]:bins[i+1]],
                                                       output_file,
                                                       input_is_sorted, extension_list=extension_list))
            print("%i/%i files will be merged" % (merged_files, number_of_files))

            self.parallel_execute(options_list, threads=threads, runtype="cp")

            self.combine_gvcf(reference, output_file_list, output, input_is_sorted=input_is_sorted,
                              extension_list=extension_list,
                              tmp_dir=tmp_dir,
                              max_files_per_merging=max_files_per_merging, iteration=iteration+1)









#!/usr/bin/env python
import os

#TODO: rewrite as package
def spades(forward_reads,
           reverse_reads=None,
           sanger_reads=None,
           pacbio_reads=None,
           only_correction=False,
           max_threads=4,
           platform="illumina",
           gzip_corrected_reads=True,
           output_dir="spades",
           kmer_length_list=[]):
    #suitable for spades v3.0.0
    """
    SPAdes genome assembler v.3.0.0

    Usage: spades.py [options] -o <output_dir>

    Basic options:
    -o	<output_dir>	directory to store all the resulting files (required)
    --sc			this flag is required for MDA (single-cell) data
    --iontorrent		this flag is required for IonTorrent data
    --test			runs SPAdes on toy dataset
    -h/--help		prints this usage message

    Input data:
    --12	<filename>	file with interlaced forward and reverse paired-end reads
    -1	<filename>	file with forward paired-end reads
    -2	<filename>	file with reverse paired-end reads
    -s	<filename>	file with unpaired reads
    --pe<#>-12	<filename>	file with interlaced reads for paired-end library number <#> (<#> = 1,2,3,4,5)
    --pe<#>-1	<filename>	file with forward reads for paired-end library number <#> (<#> = 1,2,3,4,5)
    --pe<#>-2	<filename>	file with reverse reads for paired-end library number <#> (<#> = 1,2,3,4,5)
    --pe<#>-s	<filename>	file with unpaired reads for paired-end library number <#> (<#> = 1,2,3,4,5)
    --pe<#>-<or>	orientation of reads for paired-end library number <#> (<#> = 1,2,3,4,5; <or> = fr, rf, ff)
    --mp<#>-12	<filename>	file with interlaced reads for mate-pair library number <#> (<#> = 1,2,3,4,5)
    --mp<#>-1	<filename>	file with forward reads for mate-pair library number <#> (<#> = 1,2,3,4,5)
    --mp<#>-2	<filename>	file with reverse reads for mate-pair library number <#> (<#> = 1,2,3,4,5)
    --mp<#>-s	<filename>	file with unpaired reads for mate-pair library number <#> (<#> = 1,2,3,4,5)
    --mp<#>-<or>	orientation of reads for mate-pair library number <#> (<#> = 1,2,3,4,5; <or> = fr, rf, ff)
    --sanger	<filename>	file with Sanger reads
    --pacbio	<filename>	file with PacBio reads
    --trusted-contigs	<filename>	file with trusted contigs
    --untrusted-contigs	<filename>	file with untrusted contigs

    Pipeline options:
    --only-error-correction	runs only read error correction (without assembling)
    --only-assembler	runs only assembling (without read error correction)
    --careful		tries to reduce number of mismatches and short indels
    --continue		continue run from the last available check-point
    --restart-from	<cp>	restart run with updated options and from the specified check-point ('ec', 'as', 'k<int>', 'mc')
    --disable-gzip-output	forces error correction not to compress the corrected reads
    --disable-rr		disables repeat resolution stage of assembling

    Advanced options:
    --dataset	<filename>	file with dataset description in YAML format
    -t/--threads	<int>		number of threads
                    [default: 16]
    -m/--memory	<int>		RAM limit for SPAdes in Gb (terminates if exceeded)
                    [default: 250]
    --tmp-dir	<dirname>	directory for temporary files
                    [default: <output_dir>/tmp]
    -k		<int,int,...>	comma-separated list of k-mer sizes (must be odd and
                    less than 128) [default: 'auto']
    --phred-offset	<33 or 64>	PHRED quality offset in the input reads (33 or 64)
                    [default: auto-detect]
    """

    os.system("mkdir -p %s" % output_dir)

    if platform == "illumina":
        platform = ""
    elif platform == "ion_torrent":
        platform = "--iontorrent"

    if reverse_reads:
        reads = "-1 %s -2 %s" % (forward_reads, reverse_reads)
    else:
        reads = "-s %s" % forward_reads

    if sanger_reads:
        reads += " --sanger %s" % sanger_reads
    if pacbio_reads:
        reads += " --pacbio %s" % pacbio_reads

    if only_correction:
        mode = " --only-error-correction"
    else:
        mode = ""

    if gzip_corrected_reads:
        gzip_output = ""
    else:
        gzip_output = "--disable-gzip-output"

    kmers = ""
    if kmer_length_list:
        kmers = "-k " + ",".join(map(lambda x: str(x), kmer_length_list))

    os.system("spades.py -t %i %s %s %s %s --careful %s -o %s"
              % (max_threads, mode, platform, gzip_output, reads, kmers, output_dir))

    if not gzip_corrected_reads:
        os.chdir("%s/corrected" % output_dir)
        os.system("fastqc -t 2 --nogroup *.f*q")
        os.system("cd ../..")


#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'

import os
import shutil
from RouToolPa.Tools.Abstract import Tool
from RouToolPa.Parsers.TRF import CollectionTRF
from RouToolPa.Routines import FileRoutines, AnnotationsRoutines
from RouToolPa.Collections.General import SynDict


class TRF(Tool):
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "trf", path=path, max_threads=max_threads)

    @staticmethod
    def parse_common_options(matching_weight=2, mismatching_penalty=7, indel_penalty=7,
                             match_probability=80, indel_probability=10, min_alignment_score=50, max_period=500,
                             report_flanking_sequences=False, make_dat_file=True, max_repeat_length=None,
                             suppress_html_output=False):

        options = " %i" % matching_weight
        options += " %i" % mismatching_penalty
        options += " %i" % indel_penalty
        options += " %i" % match_probability
        options += " %i" % indel_probability
        options += " %i" % min_alignment_score
        options += " %i" % max_period
        options += " -f" if report_flanking_sequences else ""
        options += " -d" if make_dat_file else ""
        options += " -l %i" % max_repeat_length if max_repeat_length else ""
        options += " -h" if suppress_html_output else ""

        return options

    def search_tandem_repeats(self, query_file, matching_weight=2, mismatching_penalty=7, indel_penalty=7,
                              match_probability=80, indel_probability=10, min_alignment_score=50, max_period=500,
                              report_flanking_sequences=False, make_dat_file=True, disable_html_output=True,
                              max_repeat_length=None, suppress_html_output=True):

        options = " %s" % query_file
        options += self.parse_common_options(matching_weight=matching_weight, mismatching_penalty=mismatching_penalty,
                                             indel_penalty=indel_penalty, match_probability=match_probability,
                                             indel_probability=indel_probability, min_alignment_score=min_alignment_score,
                                             max_period=max_period, report_flanking_sequences=report_flanking_sequences,
                                             make_dat_file=make_dat_file, max_repeat_length=max_repeat_length,
                                             suppress_html_output=suppress_html_output)

        options += " -h" if disable_html_output else ""

        self.execute(options)

    @staticmethod
    def convert_trf_report(trf_report, output_prefix):

        trf_collection = CollectionTRF(trf_file=trf_report, from_file=True)

        trf_collection.write("%s.rep" % output_prefix)
        trf_collection.write_gff("%s.gff" % output_prefix)
        trf_collection.write_gff_with_rep_seqs("%s.with_rep_seqs.gff" % output_prefix)
        trf_collection.write_simple_gff("%s.simple.gff" % output_prefix)
        trf_collection.write_short_table("%s.short.tab" % output_prefix)
        trf_collection.write_wide_table("%s.wide.tab" % output_prefix)
        trf_collection.write_fasta("%s.fasta" % output_prefix)
        trf_collection.write_bed("%s.simple.bed" % output_prefix)

        return trf_collection

    def parallel_search_tandem_repeat(self, query_file, output_prefix, matching_weight=2, mismatching_penalty=7,
                                      indel_penalty=7,
                                      match_probability=80, indel_probability=10, min_alignment_score=50, max_period=500,
                                      report_flanking_sequences=False, splited_fasta_dir="splited_fasta_dir",
                                      splited_result_dir="splited_output", converted_output_dir="converted_output",
                                      max_len_per_file=100000, store_intermediate_files=False, max_repeat_length=None):
        work_dir = os.getcwd()
        splited_filename = FileRoutines.split_filename(query_file)
        self.split_fasta_by_seq_len(query_file, splited_fasta_dir, max_len_per_file=max_len_per_file,
                                    output_prefix=splited_filename[1])

        common_options = self.parse_common_options(matching_weight=matching_weight,
                                                   mismatching_penalty=mismatching_penalty,
                                                   indel_penalty=indel_penalty, match_probability=match_probability,
                                                   indel_probability=indel_probability,
                                                   min_alignment_score=min_alignment_score,
                                                   max_period=max_period,
                                                   report_flanking_sequences=report_flanking_sequences,
                                                   make_dat_file=True, max_repeat_length=max_repeat_length)
        common_options += " -h"  # suppress html output
        options_list = []
        splited_files = os.listdir(splited_fasta_dir)

        FileRoutines.safe_mkdir(splited_result_dir)
        FileRoutines.safe_mkdir(converted_output_dir)
        os.chdir(splited_result_dir)

        input_dir = splited_fasta_dir if (splited_fasta_dir[0] == "/") or (splited_fasta_dir[0] == "~") \
                    else "../%s" % splited_fasta_dir

        for filename in splited_files:
            file_options = "%s/%s" % (input_dir, filename)
            file_options += common_options
            options_list.append(file_options)

        self.parallel_execute(options_list)

        os.chdir(work_dir)
        trf_output_file_list = []
        for filename in splited_files:

            trf_output_file = "%s/%s.%i.%i.%i.%i.%i.%i.%i.dat" % (splited_result_dir, filename,
                                                                  matching_weight, mismatching_penalty,
                                                                  indel_penalty, match_probability,
                                                                  indel_probability,
                                                                  min_alignment_score, max_period)
            trf_output_file_list.append(trf_output_file)

        trf_report = self.convert_trf_report(trf_output_file_list, output_prefix)
        """
        for suffix in (".rep", ".gff", ".simple.gff", ".short.tab", ".wide.tab", ".with_rep_seqs.gff", ".fasta"):
            file_str = ""
            merged_file = "%s%s" % (output_prefix, suffix)
            for filename in splited_files:
                file_str += " %s/%s%s" % (converted_output_dir, filename, suffix)
            CGAS.cat(file_str, merged_file)
        """
        compress_splited_out_string = "tar czf %s.splited_output.tar.gz %s" % (output_prefix, splited_result_dir)
        os.system(compress_splited_out_string)

        if not store_intermediate_files:
            shutil.rmtree(splited_fasta_dir)
            shutil.rmtree(splited_result_dir)
            shutil.rmtree(converted_output_dir)

        return trf_report

    def parallel_search_long_tandem_repeat(self, query_file, output_prefix, matching_weight=2, mismatching_penalty=3,
                                           indel_penalty=5,
                                           match_probability=80, indel_probability=10, min_alignment_score=50, max_period=2000,
                                           report_flanking_sequences=False, splited_fasta_dir="splited_fasta_dir",
                                           splited_result_dir="splited_output", converted_output_dir="converted_output",
                                           max_len_per_file=100000, store_intermediate_files=False, max_repeat_length=6):
        self.parallel_search_tandem_repeat(query_file, output_prefix, matching_weight=matching_weight,
                                           mismatching_penalty=mismatching_penalty,
                                           indel_penalty=indel_penalty,
                                           match_probability=match_probability,
                                           indel_probability=indel_probability,
                                           min_alignment_score=min_alignment_score, max_period=max_period,
                                           report_flanking_sequences=report_flanking_sequences,
                                           splited_fasta_dir=splited_fasta_dir,
                                           splited_result_dir=splited_result_dir,
                                           converted_output_dir=converted_output_dir,
                                           max_len_per_file=max_len_per_file,
                                           store_intermediate_files=store_intermediate_files,
                                           max_repeat_length=max_repeat_length)

    @staticmethod
    def gff_filtering_expression(gff_description_dict, min_period=None, max_period=None, min_copy_number=None,
                                 max_copy_number=None, pattern=None, min_percentage_of_matches=None,
                                 max_percentage_of_indels=None, min_entropy=None, max_entropy=None):
        """

        """

        """
        print "period:", min_period, max_period
        print "copy:", min_copy_number, max_copy_number
        print "pattern", pattern
        print gff_description_dict
        """
        if not (min_period is None):
            if int(gff_description_dict["Period"]) < min_period:
                return False
        if not (max_period is None):
            if int(gff_description_dict["Period"]) > max_period:
                return False
        if not (min_copy_number is None):
            if float(gff_description_dict["N_copies"]) < min_copy_number:
                return False
        if not (max_copy_number is None):
            if float(gff_description_dict["N_copies"]) > max_copy_number:
                return False
        if not (pattern is None):
            if gff_description_dict["Pattern"] != pattern:
                return False
        if not (min_percentage_of_matches is None):
            if int(gff_description_dict["Pers_matches"]) < min_percentage_of_matches:
                return False
        if not (max_percentage_of_indels is None):
            if int(gff_description_dict["Pers_indels"]) > max_percentage_of_indels:
                return False
        if not (min_entropy is None):
            if float(gff_description_dict["Entropy"]) < min_entropy:
                return False
        if not (max_entropy is None):
            if float(gff_description_dict["Entropy"]) > max_entropy:
                return False

        return True

    def filter_trf_gff(self, input_gff, output_gff, filtered_out_gff, min_period=None, max_period=None,
                       min_copy_number=None,
                       max_copy_number=None, pattern=None, min_percentage_of_matches=None,
                       max_percentage_of_indels=None, min_entropy=None, max_entropy=None):

        def filtering_expression(gff_description_dict):
            return self.gff_filtering_expression(gff_description_dict, min_period=min_period, max_period=max_period,
                                                 min_copy_number=min_copy_number,
                                                 max_copy_number=max_copy_number,
                                                 pattern=pattern,
                                                 min_percentage_of_matches=min_percentage_of_matches,
                                                 max_percentage_of_indels=max_percentage_of_indels,
                                                 min_entropy=min_entropy,
                                                 max_entropy=max_entropy)

        AnnotationsRoutines.filter_gff_by_description(input_gff, output_gff, filtered_out_gff, filtering_expression)

    @staticmethod
    def get_monomer_len_file_from_trf_gff(trf_gff, len_file):
        len_dict = SynDict()

        with open(trf_gff, "r") as trf_fd:
            for line in trf_fd:
                if line[0] == "#":
                    continue
                description_dict = AnnotationsRoutines.get_description_dict_from_gff_string(line)
                len_dict[description_dict["ID"]] = description_dict["Period"]
        # print len_dict
        len_dict.write(len_file)

    @staticmethod
    def filter_trf_gff_by_exact_copy_number(input_gff, output_gff, filtered_out_gff, min_copy_number,
                                            perfect_tandem=False):

        if perfect_tandem:
            def filtering_expression(gff_description_dict):
                if (gff_description_dict["Pattern"] * min_copy_number) in gff_description_dict["seq"]:
                    return True
                return False
        else:
            def filtering_expression(gff_description_dict):

                if gff_description_dict["seq"].count(gff_description_dict["Pattern"]) >= min_copy_number:
                    return True
                return False

        AnnotationsRoutines.filter_gff_by_description(input_gff, output_gff, filtered_out_gff, filtering_expression)


if __name__ == "__main__":
    pass



#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'

from RouToolPa.Tools.Abstract import Tool


class EMPHF(Tool):
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "mphf", path=path, max_threads=max_threads)

    def compute_mphf_seq(self, kmer_file, pf_file):
        """

        :param kmer_file: input file with kmer list
        :param pf_file: pf file to create
        :return:

        creates pf_file

        """
        options = " %s" % kmer_file
        options += " %s" % pf_file

        self.execute(options=options, cmd="compute_mphf_seq")



#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'

import os
from RouToolPa.Tools.Abstract import Tool


class Quast(Tool):
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "quast.py", path=path, max_threads=max_threads)

    def get_stat(self, contigs_file, out_dir="quast", reference=None, genes_file=None, operon_file=None, min_contig_length=None,
                 predict_genes=False, eukaryotic_genome=True, metagenome=False, scaffolds=False):
        options = " -T %i" % self.threads
        options += " -o %s" % out_dir
        options += " -R %s" % reference if reference else ""
        options += " -G %s" % genes_file if genes_file else ""
        options += " -O %s" % operon_file if operon_file else ""
        options += " --min-contig" % min_contig_length if min_contig_length else ""
        options += " -f" if predict_genes else ""
        options += " -e" if eukaryotic_genome else ""
        options += " -m" if metagenome else ""
        options += " -s" if scaffolds else ""

        options += " %s" % contigs_file
        self.execute(options)


if __name__ == "__main__":
    pass


#!/usr/bin/env python
__author__ = 'mahajrod'

from RouToolPa.Tools.Abstract import Tool


class BLAST(Tool):
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "blastall", path=path, max_threads=max_threads)

    def make_db(self, input_files, title, input_type="nucleotide", logfile=None, parse_seqid=None, seq_entry_input=None,
                blast_base_names=None):
        """
      -t  Title for database file [String]  Optional
      -i  Input file(s) for formatting [File In]  Optional
      -l  Logfile name: [File Out]  Optional
        default = formatdb.log
      -p  Type of file
             T - protein
             F - nucleotide [T/F]  Optional
        default = T
      -o  Parse options
             T - True: Parse SeqId and create indexes.
             F - False: Do not parse SeqId. Do not create indexes.
     [T/F]  Optional
        default = F
      -a  Input file is database in ASN.1 format (otherwise FASTA is expected)
             T - True,
             F - False.
     [T/F]  Optional
        default = F
      -b  ASN.1 database in binary mode
             T - binary,
             F - text mode.
     [T/F]  Optional
        default = F
      -e  Input is a Seq-entry [T/F]  Optional
        default = F
      -n  Base name for BLAST files [String]  Optional
      -v  Database volume size in millions of letters [Integer]  Optional
        default = 4000
      -s  Create indexes limited only to accessions - sparse [T/F]  Optional
        default = F
      -V  Verbose: check for non-unique string ids in the database [T/F]  Optional
        default = F
      -L  Create an alias file with this name
            use the gifile arg (below) if set to calculate db size
            use the BLAST db specified with -i (above) [File Out]  Optional
      -F  Gifile (file containing list of gi's) [File In]  Optional
      -B  Binary Gifile produced from the Gifile specified above [File Out]  Optional
      -T  Taxid file to set the taxonomy ids in ASN.1 deflines [File In]  Optional
        """
        options = ""
        options += " -t %s" % title
        options += " -l %s" % logfile if logfile else ""
        options += " -p %s" % ("T" if input_type == "protein" else "F")
        options += " -o T" if parse_seqid else ""
        options += " -e T" if seq_entry_input else ""
        options += " -n %s" % blast_base_names if blast_base_names else ""
        # checking if inputfiles has method __iter__
        options += " -i %s" % (" ".join(input_files) if hasattr(input_files, '__iter__') and not isinstance(input_files, str) else input_files)

        #TODO: add parsing of rest of arguments
        self.execute(options, cmd="formatdb")


if __name__ == "__main__":
    pass

    """
    workdir = "/home/mahajrod/genetics/reference/"
    blast = BLAST()

    os.chdir(workdir)
    files = os.listdir("./")

    links_directory = "/home/mahajrod/genetics/nxf/annotation/test/Dmel/nxf1/"
    drosophila_dirs = []
    for entry in files:
        if "drosophila" in entry:
            drosophila_dirs.append(entry)

    for directory in drosophila_dirs:
        os.chdir(workdir)
        os.chdir(directory)
        reference = os.listdir("./")[0] + "/"
        os.chdir(reference)
        os.chdir("ref")
        for file_entry in os.listdir("./"):
            if file_entry[-6:] == ".fasta":
                reference_file = file_entry
        blast.make_db(reference_file, directory, parse_seqid=True)
        #print("ln -s %s %s" % (workdir + directory + "/ref/" + reference_file, links_directory))
        #os.system("ln -fs %s %s" % (workdir + directory + "/" + reference + "ref/" + reference_file, links_directory))
    """


__author__ = 'mahajrod'
import sys

if sys.version_info[0] == 2:
    from ete2 import Tree
else:
    from ete3 import Tree


class TreeRoutines:

    @staticmethod
    def unroot_tree_from_file(input_tree_file, output_tree_file, input_tree_format, output_tree_format=None):
        with open(input_tree_file, "r") as in_fd:
            with open(output_tree_file, "w") as out_fd:
                for line in in_fd:
                    tree_line = line.strip()
                    tree = Tree(tree_line, format=input_tree_format)
                    tree.unroot()
                    out_fd.write(tree.write(format=output_tree_format if output_tree_format
                                                                    else input_tree_format) + "\n")



__author__ = 'mahajrod'

from RouToolPa.Tools.GATK4.SortVcf4 import SortVcf4
from RouToolPa.Tools.GATK4.CombineGVCFs4 import CombineGVCFs4
from RouToolPa.Tools.GATK4.GenotypeGVCFs4 import GenotypeGVCFs4
from RouToolPa.Tools.GATK4.SelectVariants4 import SelectVariants4
#from RouToolPa.Tools.GATK4.CombineVariants4 import CombineVariants4
from RouToolPa.Tools.GATK4.HaplotypeCaller4 import HaplotypeCaller4
from RouToolPa.Tools.GATK4.GenomicsDBImport4 import GenomicsDBImport4
from RouToolPa.Tools.GATK4.ValidateVariants4 import ValidateVariants4
from RouToolPa.Tools.GATK4.BaseRecalibrator4 import BaseRecalibrator4
#from RouToolPa.Tools.GATK4.UnifiedGenotyper4 import UnifiedGenotyper4
#from RouToolPa.Tools.GATK4.AnalyzeCovariates4 import AnalyzeCovariates4
from RouToolPa.Tools.GATK4.VariantFiltration4 import VariantFiltration4
#from RouToolPa.Tools.GATK4.FastaAlternateReferenceMaker4 import FastaAlternateReferenceMaker4

SortVcf4 = SortVcf4()
CombineGVCFs4 = CombineGVCFs4()
GenotypeGVCFs4 = GenotypeGVCFs4()
SelectVariants4 = SelectVariants4()
#CombineVariants4 = CombineVariants4()
HaplotypeCaller4 = HaplotypeCaller4()
GenomicsDBImport4 = GenomicsDBImport4()
ValidateVariants4 = ValidateVariants4()
#BaseRecalibrator4 = BaseRecalibrator4()
#UnifiedGenotyper4 = UnifiedGenotyper4()
#AnalyzeCovariates4 = AnalyzeCovariates4()
VariantFiltration4 = VariantFiltration4()
#FastaAlternateReferenceMaker4 = FastaAlternateReferenceMaker4()



#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'

from RouToolPa.Tools.Assemblers.BionanoSolve import BionanoSolve
from RouToolPa.Tools.Assemblers.Quast import Quast
from RouToolPa.Tools.Assemblers.Spades import Spades
from RouToolPa.Tools.Assemblers.MaSuRCA import MaSuRCA
from RouToolPa.Tools.Assemblers.Supernova import Supernova


Quast = Quast()
Spades = Spades()
MaSuRCA = MaSuRCA()
Supernova = Supernova()
BionanoSolve = BionanoSolve()



#!/usr/bin/env python

__author__ = 'mahajrod'

from RouToolPa.Tools.Abstract import Tool


class GATKTool(Tool):

    def __init__(self, java_path="", max_threads=4, jar_path="", max_memory="1g"):

        jar = "GenomeAnalysisTK.jar"

        Tool.__init__(self, "java", path=java_path, max_threads=max_threads, jar_path=jar_path, jar=jar, max_memory=max_memory)



#!/usr/bin/env python

from RouToolPa.Tools.Abstract import Tool


class PartitionFinder(Tool):
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "partitionfinder", path=path, max_threads=max_threads)

    #TODO: modify followng function
    @staticmethod
    def generate_partition_finder_control_file(alignment_file_name,
                                               genes_file,
                                               output_dir=None,
                                               genes_names_list=None,
                                               branchlengths="linked",
                                               models="raxml",
                                               model_selection="AIC",
                                               search="greedy"):
        #supported models: all, raxml, mrbayes, beast
        #supported model_selection: AIC, AICc, BIC
        #supported search: all, greedy, rcluster, hcluster, user
        print("Generating config file for PartionFinder")
        output_filename = "partition_finder.cfg"
        if output_dir:
            output_filename = output_dir + "/partition_finder.cfg"

        fd_genes = open(genes_file, "r")
        fd_genes.readline()
        coordinates_list = []
        for line in fd_genes:
            line_list = [int(x) for x in line.strip().split("\t")]
            coordinates_list.append(line_list)
        if not line_list[-1]:
            del(line_list[-1])
        fd_genes.close()

        fd = open(output_filename, "w")
        fd.write("""
    ## ALIGNMENT FILE ##
    alignment = %s;

    ## BRANCHLENGTHS: linked | unlinked ##
    branchlengths = %s;

    ## MODELS OF EVOLUTION for PartitionFinder: all | raxml | mrbayes | beast | <list> ##
    ##              for PartitionFinderProtein: all_protein | <list> ##
    models = %s;

    # MODEL SELECCTION: AIC | AICc | BIC #
    model_selection = %s;

    ## DATA BLOCKS: see manual for how to define ##
    [data_blocks]\n
    """ % (alignment_file_name, branchlengths, models, model_selection))

        for i in range(0, len(coordinates_list)):
            name = "part%i" % (i + 1)
            if genes_names_list:
                name = genes_names_list[i]
            fd.write("%s_pos1 = %i-%i\\3;\n" % (name, coordinates_list[i][1], coordinates_list[i][2]))
            fd.write("%s_pos2 = %i-%i\\3;\n" % (name, coordinates_list[i][1] + 1, coordinates_list[i][2]))
            fd.write("%s_pos3 = %i-%i\\3;\n" % (name, coordinates_list[i][1] + 2, coordinates_list[i][2]))
        fd.write("""
    ## SCHEMES, search: all | greedy | rcluster | hcluster | user ##
    [schemes]
    search = %s;

    #user schemes go here if search=user. See manual for how to define.#
    """ % search)
        fd.close()


#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'
from math import ceil
from RouToolPa.Tools.Abstract import Tool
from RouToolPa.Collections.General import SynDict, TwoLvlDict



class Subread(Tool):
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "featureCounts", path=path, max_threads=max_threads)

    def count(self, alignment_file, gff_file, output_file, count_multimapped_reads=False, annotation_file_type="GTF",
              min_read_fraction_overlap=None, feature_type_to_use=None, attribute_type_to_use=None, stranded=None):
        """
          Usage: featureCounts [options] -a <annotation_file> -o <output_file> input_file1 [input_file2] ...

## Mandatory arguments:

  -a <string>         Name of an annotation file. GTF/GFF format by default.
                      See -F option for more format information. Inbuilt
                      annotations (SAF format) is available in 'annotation'
                      directory of the package.

  -o <string>         Name of the output file including read counts. A separate
                      file including summary statistics of counting results is
                      also included in the output ('<string>.summary')

  input_file1 [input_file2] ...   A list of SAM or BAM format files. They can be
                      either name or location sorted. If not files provided,
                      <stdin> input is expected.

## Optional arguments:
# Annotation

  -F <string>         Specify format of the provided annotation file. Acceptable
                      formats include 'GTF' (or compatible GFF format) and
                      'SAF'. 'GTF' by default.  For SAF format, please refer to
                      Users Guide.

  -t <string>         Specify feature type in GTF annotation. 'exon' by
                      default. Features used for read counting will be
                      extracted from annotation using the provided value.

  -g <string>         Specify attribute type in GTF annotation. 'gene_id' by
                      default. Meta-features used for read counting will be
                      extracted from annotation using the provided value.

  -A <string>         Provide a chromosome name alias file to match chr names in
                      annotation with those in the reads. This should be a two-
                      column comma-delimited text file. Its first column should
                      include chr names in the annotation and its second column
                      should include chr names in the reads. Chr names are case
                      sensitive. No column header should be included in the
                      file.

# Level of summarization

  -f                  Perform read counting at feature level (eg. counting
                      reads for exons rather than genes).

# Overlap between reads and features

  -O                  Assign reads to all their overlapping meta-features (or
                      features if -f is specified).

  --minOverlap <int>  Minimum number of overlapping bases in a read that is
                      required for read assignment. 1 by default. Number of
                      overlapping bases is counted from both reads if paired
                      end. If a negative value is provided, then a gap of up
                      to specified size will be allowed between read and the
                      feature that the read is assigned to.

  --fracOverlap <float> Minimum fraction of overlapping bases in a read that is
                      required for read assignment. Value should be within range
                      [0,1]. 0 by default. Number of overlapping bases is
                      counted from both reads if paired end. Both this option
                      and '--minOverlap' option need to be satisfied for read
                      assignment.

  --largestOverlap    Assign reads to a meta-feature/feature that has the
                      largest number of overlapping bases.

  --readExtension5 <int> Reads are extended upstream by <int> bases from their
                      5' end.

  --readExtension3 <int> Reads are extended upstream by <int> bases from their
                      3' end.

  --read2pos <5:3>    Reduce reads to their 5' most base or 3' most base. Read
                      counting is then performed based on the single base the
                      read is reduced to.

# Multi-mapping reads

  -M                  Multi-mapping reads will also be counted. For a multi-
                      mapping read, all its reported alignments will be
                      counted. The 'NH' tag in BAM/SAM input is used to detect
                      multi-mapping reads.

# Fractional counting

  --fraction          Assign fractional counts to features. This option must
                      be used together with '-M' or '-O' or both. When '-M' is
                      specified, each reported alignment from a multi-mapping
                      read (identified via 'NH' tag) will carry a fractional
                      count of 1/x, instead of 1 (one), where x is the total
                      number of alignments reported for the same read. When '-O'
                      is specified, each overlapping feature will receive a
                      fractional count of 1/y, where y is the total number of
                      features overlapping with the read. When both '-M' and
                      '-O' are specified, each alignment will carry a fractional
                      count of 1/(x*y).

# Read filtering

  -Q <int>            The minimum mapping quality score a read must satisfy in
                      order to be counted. For paired-end reads, at least one
                      end should satisfy this criteria. 0 by default.

  --splitOnly         Count split alignments only (ie. alignments with CIGAR
                      string containing 'N'). An example of split alignments is
                      exon-spanning reads in RNA-seq data.

  --nonSplitOnly      If specified, only non-split alignments (CIGAR strings do
                      not contain letter 'N') will be counted. All the other
                      alignments will be ignored.

  --primary           Count primary alignments only. Primary alignments are
                      identified using bit 0x100 in SAM/BAM FLAG field.

  --ignoreDup         Ignore duplicate reads in read counting. Duplicate reads
                      are identified using bit Ox400 in BAM/SAM FLAG field. The
                      whole read pair is ignored if one of the reads is a
                      duplicate read for paired end data.

# Strandness

  -s <int>            Perform strand-specific read counting. Acceptable values:
                      0 (unstranded), 1 (stranded) and 2 (reversely stranded).
                      0 by default.

# Exon-exon junctions

  -J                  Count number of reads supporting each exon-exon junction.
                      Junctions were identified from those exon-spanning reads
                      in the input (containing 'N' in CIGAR string). Counting
                      results are saved to a file named '<output_file>.jcounts'

  -G <string>         Provide the name of a FASTA-format file that contains the
                      reference sequences used in read mapping that produced the
                      provided SAM/BAM files. This optional argument can be used
                      with '-J' option to improve read counting for junctions.

# Parameters specific to paired end reads

  -p                  If specified, fragments (or templates) will be counted
                      instead of reads. This option is only applicable for
                      paired-end reads.

  -B                  Only count read pairs that have both ends aligned.

  -P                  Check validity of paired-end distance when counting read
                      pairs. Use -d and -D to set thresholds.

  -d <int>            Minimum fragment/template length, 50 by default.

  -D <int>            Maximum fragment/template length, 600 by default.

  -C                  Do not count read pairs that have their two ends mapping
                      to different chromosomes or mapping to same chromosome
                      but on different strands.

  --donotsort         Do not sort reads in BAM/SAM input. Note that reads from
                      the same pair are required to be located next to each
                      other in the input.

# Number of CPU threads

  -T <int>            Number of the threads. 1 by default.

# Miscellaneous

  -R                  Output detailed assignment result for each read. A text
                      file will be generated for each input file, including
                      names of reads and meta-features/features reads were
                      assigned to. See Users Guide for more details.

  --tmpDir <string>   Directory under which intermediate files are saved (later
                      removed). By default, intermediate files will be saved to
                      the directory specified in '-o' argument.

  --maxMOp <int>      Maximum number of 'M' operations allowed in a CIGAR
                      string. 10 by default. Both 'X' and '=' are treated as 'M'
                      and adjacent 'M' operations are merged in the CIGAR
                      string.

  -v                  Output version of the program.

        """

        options = " -T %i" % self.threads

        options += " -M" if count_multimapped_reads else ""
        options += " --fracOverlap %f" % min_read_fraction_overlap if min_read_fraction_overlap else ""
        options += " -F %s" % annotation_file_type

        options += " -t %s" % feature_type_to_use if feature_type_to_use else ""
        options += " -g %s" % attribute_type_to_use if attribute_type_to_use else ""
        options += " -F %s" % annotation_file_type

        options += " -a %s" % gff_file
        options += " -o %s" % output_file
        options += " %s" % alignment_file
        options += " -s %i" % stranded if stranded else ""

        self.execute(options=options)

    def count_miRNA_reads(self, alignment_file, gff_file, output_prefix, annotation_file_type="GTF",
                          min_read_fraction_overlap=1.0, feature_type_to_use=None, attribute_type_to_use=None,
                          sample_name=None, stranded=1):

        no_multimapped_read_counts = "%s.no_multimapped_reads.count" % output_prefix
        with_multimapped_read_counts = "%s.with_multimapped_reads.count" % output_prefix
        all_adjusted_read_counts = "%s.all_adjusted_reads.count" % output_prefix

        self.count(alignment_file, gff_file, no_multimapped_read_counts, annotation_file_type=annotation_file_type,
                   min_read_fraction_overlap=min_read_fraction_overlap, feature_type_to_use=feature_type_to_use,
                   attribute_type_to_use=attribute_type_to_use, stranded=stranded)

        self.count(alignment_file, gff_file, with_multimapped_read_counts, count_multimapped_reads=True,
                   annotation_file_type=annotation_file_type,
                   min_read_fraction_overlap=min_read_fraction_overlap, feature_type_to_use=feature_type_to_use,
                   attribute_type_to_use=attribute_type_to_use, stranded=stranded)

        no_multimapped_read_count_dict = SynDict(filename=no_multimapped_read_counts, comments_prefix="#",
                                                 key_index=0, value_index=6, expression=int, header=True)
        with_multimapped_read_count_dict = SynDict(filename=with_multimapped_read_counts, comments_prefix="#",
                                                   key_index=0, value_index=6, expression=int, header=True)
        similar_feature_number_dict = SynDict(filename=with_multimapped_read_counts, comments_prefix="#", header=True,
                                              key_index=0, value_index=1, expression=lambda s: len(s.split(";")))

        sample_nameeee = sample_name if sample_name else similar_feature_number_dict.header.split()[6]

        all_adjusted_read_count_dict = SynDict()
        all_adjusted_read_count_dict.header = ".\t%s" % sample_nameeee

        #print no_multimapped_read_count_dict
        #print with_multimapped_read_count_dict
        #print similar_feature_number_dict

        for feature_id in no_multimapped_read_count_dict:
            all_adjusted_read_count_dict[feature_id] = int(ceil(float(no_multimapped_read_count_dict[feature_id]) + \
                                                            (float(with_multimapped_read_count_dict[feature_id]) - float(no_multimapped_read_count_dict[feature_id])) / float(similar_feature_number_dict[feature_id])))

        all_adjusted_read_count_dict.write(all_adjusted_read_counts, header=True)









#!/usr/bin/env python

import os
from RouToolPa.Tools.Abstract import Tool
from RouToolPa.Routines import DrawingRoutines




class SamtoolsV1(Tool):
    """
    Class for samtools 1.0+
    Several subcommands are not implemented
    """
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "samtools", path=path, max_threads=max_threads)

        # bam/sam flag values:
        self.bam_flags = {
                          "read_paired": 1,
                          "read_mapped_in_proper_pair": 2,
                          "read_unmapped": 4,
                          "mate_unmapped": 8,
                          "read_reverse_strand": 16,
                          "mate_reverse_strand": 32,
                          "first_in_pair": 64,
                          "second_in_pair": 128,
                          "not_primary_alignment": 256,
                          "read_fails_platform/vendor_quality_checks": 512,
                          "read_is_PCR_or_optical_duplicate": 1024,
                          "supplementary_alignment": 2048
                          }

    def get_read_names(self, input_bam, output_file):

        options = " view"
        options += " %s" % input_bam
        options += " | awk \'{print $1}\' > %s" % output_file

        self.execute(options=options)

    def get_reads_by_name(self, read_name_list, input_sam_fd, output_sam_fd, mode="include", search_mode="exact"):

        if search_mode == "exact":
            def expression(read_name):
                for entry in read_name_list:
                    if entry == read_name:
                        return True
                return False
        elif search_mode == "partial":
            def expression(read_name):
                for entry in read_name_list:
                    if entry in read_name:
                        return True
                return False

        if mode == "include":
            def include_expression(read_name):
                return expression(read_name)
        elif mode == "remove":
            def include_expression(read_name):
                return not expression(read_name)

        for line in input_sam_fd:
            if line[0] != "@":
                if include_expression(line.split()[0]):
                    output_sam_fd.write(line)
                break
            output_sam_fd.write(line)
        for line in input_sam_fd:
            if include_expression(line.split()[0]):
                output_sam_fd.write(line)

    def rmdup(self, input_bam, output_bam, remove_dup_for_se_reads=False, treat_both_pe_and_se_reads=False):

        options = " -s" if remove_dup_for_se_reads else ""
        options += " -S" if treat_both_pe_and_se_reads else ""
        options += " %s" % input_bam
        options += " %s" % output_bam

        self.execute(options, cmd="samtools rmdup")

    def parse_view_options(self, include_header_in_output=True, output_uncompressed_bam=False,
                           output_bam=False, white_flag_value=None, black_flag_value=None,
                           bed_file_with_regions_to_output=None):
        options = " -@ %i" % self.threads
        options += " -h" if include_header_in_output else ""
        options += " -u" if output_uncompressed_bam else ""
        options += " -b" if output_bam else ""
        options += " -f %i" % white_flag_value if white_flag_value else ""
        options += " -F %i" % black_flag_value if black_flag_value else ""
        options += " -L %s" % bed_file_with_regions_to_output if bed_file_with_regions_to_output else ""
        return options

    def view(self, input_file, output_file, include_header_in_output=True, output_uncompressed_bam=False,
             output_bam=False, white_flag_value=None, black_flag_value=None, bed_file_with_regions_to_output=None):

        options = self.parse_view_options(include_header_in_output=include_header_in_output,
                                          output_uncompressed_bam=output_uncompressed_bam,
                                          output_bam=output_bam, white_flag_value=white_flag_value,
                                          black_flag_value=black_flag_value,
                                          bed_file_with_regions_to_output=bed_file_with_regions_to_output)

        options += " -o %s" % output_file
        options += " %s" % input_file

        self.execute(options, "samtools view")

    def parse_sort_options(self, temp_file_prefix="temp_bam", sort_by_name=False, max_memory_per_thread="1G"):
        options = " -@ %i" % self.threads
        options += " -n" if sort_by_name else ""
        options += " -m %s" % max_memory_per_thread if max_memory_per_thread else ""
        options += " -T %s" % temp_file_prefix
        return options

    def sort(self, input_bam, output_bam, temp_file_prefix="temp_bam", sort_by_name=False, max_memory_per_thread="1G"):
        options = self.parse_sort_options(temp_file_prefix=temp_file_prefix, sort_by_name=sort_by_name,
                                          max_memory_per_thread=max_memory_per_thread)
        options += " -o %s" % output_bam
        options += " %s" % input_bam

        self.execute(options, "samtools sort")

    def get_info_from_flags(self, flag_list):
        info_field = 0
        for flag in flag_list:
            info_field |= self.bam_flags[flag]

        return info_field

    def prepare_bam_for_read_extraction(self, input_bam, output_bam, temp_file_prefix="temp_bam",
                                        max_memory_per_thread="1G", bam_file_to_write_unpaired_reads=None):
        black_list_flags = ["not_primary_alignment", "supplementary_alignment"]
        info_field = self.get_info_from_flags(black_list_flags)

        view_options = " -h -b -u -F %i %s" % (info_field, input_bam)
        sort_options = " -@ %i -n -m %s -T %s -o %s -" % (self.threads, max_memory_per_thread,
                                                          temp_file_prefix, output_bam)

        if bam_file_to_write_unpaired_reads:
            split_unpaired_options = " -h -f %i -U %s" % (self.bam_flags["read_paired"], bam_file_to_write_unpaired_reads)
            cmd = "samtools view %s | samtools view %s | samtools sort %s" % (view_options, split_unpaired_options,
                                                                              sort_options)
        else:
            cmd = "samtools view %s | samtools sort %s" % (view_options, sort_options)
        self.execute(options="", cmd=cmd)

    def faidx(self, fasta_file):

        options = " %s" % fasta_file

        self.execute(options, cmd="samtools faidx")

    def check_for_fasta_index(self, fasta_file):
        if not os.path.exists(fasta_file + ".fai"):
            self.faidx(fasta_file)

    def index(self, bam_file):

        options = " %s" % bam_file

        self.execute(options=options, cmd="samtools index")

    def sam2bam(self, input_sam, output_bam, sort=True, temp_file_prefix="temp_bam",
                sort_by_name=False, max_memory_per_thread="1G"):

        options = " -bh"
        options += " -o %s" % output_bam if not sort else ""
        options += " %s" % input_sam

        sort_options = self.parse_sort_options(temp_file_prefix=temp_file_prefix, sort_by_name=sort_by_name,
                                               max_memory_per_thread=max_memory_per_thread)
        sort_options += " -o %s" % output_bam
        sort_options += " -"

        options += " | samtools sort %s" % sort_options if sort else ""

        self.execute(options=options, cmd="samtools view")

    def convert_sam_and_index(self, input_sam, output_bam, sort=True, temp_file_prefix="temp_bam",
                              sort_by_name=False, max_memory_per_thread="1G"):

        self.sam2bam(input_sam, output_bam, sort=sort, temp_file_prefix=temp_file_prefix,
                     sort_by_name=sort_by_name, max_memory_per_thread=max_memory_per_thread)
        self.index(output_bam)

    def get_insert_sizes(self, input_sam, output_prefix):

        output_concordant_only = "%s.concordant.len" % output_prefix
        output_discordant_only = "%s.discordant.len" % output_prefix
        output_all = "%s.all.len" % output_prefix

        common_flags_for_filtering_out = self.bam_flags["supplementary_alignment"]
        common_flags_for_filtering_out += self.bam_flags["not_primary_alignment"]
        common_flags_for_filtering_out += self.bam_flags["read_unmapped"]
        common_flags_for_filtering_out += self.bam_flags["mate_unmapped"]

        flags_for_discordant_only = common_flags_for_filtering_out + self.bam_flags["read_mapped_in_proper_pair"]

        samtools_options_for_discordant_only = " -F %i" % flags_for_discordant_only
        samtools_options_for_discordant_only += " %s" % input_sam

        samtools_options_for_concordant_only = " -F %i" % common_flags_for_filtering_out
        samtools_options_for_concordant_only += " -f %i" % self.bam_flags["read_mapped_in_proper_pair"]
        samtools_options_for_concordant_only += " %s" % input_sam

        samtools_string_for_discordant_only = "samtools view %s" % samtools_options_for_discordant_only
        samtools_string_for_concordant_only = "samtools view %s" % samtools_options_for_concordant_only

        awk_string = "awk -F'\\t' '{ if ($9 > 0) print $9}'"
        cmd_discordant_only = "%s | %s > %s" % (samtools_string_for_discordant_only, awk_string, output_discordant_only)
        cmd_concordant_only = "%s | %s > %s" % (samtools_string_for_concordant_only, awk_string, output_concordant_only)
        print(cmd_discordant_only)
        print(cmd_concordant_only)

        self.parallel_execute(options_list=[cmd_concordant_only, cmd_discordant_only], cmd="", threads=2,)

        cat_cmd = "cat %s %s > %s" % (output_discordant_only, output_concordant_only, output_all)
        self.execute(cmd=cat_cmd)

    def draw_insert_size_distribution(self, input_sam, output_prefix, width_of_bin=5, max_insert_size=1200,
                                      min_insert_size=0, extensions=("png",), separator="\n", logbase=10):

        output_concordant_only_file = "%s.concordant.len" % output_prefix
        output_discordant_only_file = "%s.discordant.len" % output_prefix
        output_all_file = "%s.all.len" % output_prefix

        self.get_insert_sizes(input_sam, output_prefix)
        """
        DrawingRoutines.draw_tetra_histogram_with_two_logscaled_from_file([output_concordant_file, output_all_file],
                                                                          output_prefix, figsize=(10, 10),
                                                                          number_of_bins_list=None,
                                                                          width_of_bins_list=[width_of_bin, width_of_bin],
                                                                          max_threshold_list=[max_insert_size, max_insert_size],
                                                                          min_threshold_list=[min_insert_size, min_insert_size],
                                                                          xlabel="Insert size",
                                                                          ylabel="Number of fragments",
                                                                          title_list=["Concordant pairs", "All pairs"],
                                                                          logbase=logbase,
                                                                          label_list=None,
                                                                          extensions=extensions,
                                                                          suptitle="Insert size distribution",
                                                                          separator=separator)
        """
        DrawingRoutines.draw_hexa_histogram_with_three_logscaled_from_file([output_concordant_only_file,
                                                                           output_discordant_only_file,
                                                                           output_all_file],
                                                                           None,
                                                                           output_prefix, figsize=(15, 10),
                                                                           number_of_bins_list=None,
                                                                           width_of_bins_list=[width_of_bin,
                                                                                               width_of_bin,
                                                                                               width_of_bin],
                                                                           max_threshold_list=[max_insert_size,
                                                                                               max_insert_size,
                                                                                               max_insert_size],
                                                                           min_threshold_list=[min_insert_size,
                                                                                               min_insert_size,
                                                                                               min_insert_size],
                                                                           xlabel="Insert size",
                                                                           ylabel="Number of fragments",
                                                                           title_list=["Concordant pairs",
                                                                                       "Discordant pairs",
                                                                                       "All pairs"],
                                                                           logbase=logbase,
                                                                           label_list=None,
                                                                           extensions=extensions,
                                                                           suptitle="Insert size distribution",
                                                                           separator=separator)

    def remove_duplicates_and_poorly_aligned_reads(self, input_bam=None, output_bam=None, min_mapping_quality=None):

        options = " -b"
        options += " -@ %i" % self.threads
        options += " -q %i" % min_mapping_quality if min_mapping_quality else ""
        options += " -F %i" % self.bam_flags["read_is_PCR_or_optical_duplicate"]
        options += " -o %s" % output_bam if output_bam else ""
        options += " %s" % input_bam if input_bam else " -"

        self.execute(options, cmd="samtools view ")


class SamtoolsV0(SamtoolsV1, Tool):
    """
    Class for samtools 0.1.19+
    Several subcommands are not implemented
    """

    def __init__(self, path="", max_threads=4):
        SamtoolsV1.__init__(self, path=path, max_threads=max_threads)

    def view(self, input_file, output_file, sam_input=False, include_header_in_output=True,
             output_uncompressed_bam=False, output_bam=False, white_flag_value=None, black_flag_value=None,
             bed_file_with_regions_to_output=None):

        options = " -h" if include_header_in_output else ""
        options += " -S" if sam_input else ""
        options += " -u" if output_uncompressed_bam else ""
        options += " -b" if output_bam else ""
        options += " -o %s" % output_file
        options += " -f %i" % white_flag_value if white_flag_value else ""
        options += " -F %i" % black_flag_value if black_flag_value else ""
        options += " -L %s" % bed_file_with_regions_to_output if bed_file_with_regions_to_output else ""
        options += " %s" % input_file

        self.execute(options, "samtools view")

    def sort(self, input_bam, output_prefix, max_memory=3000000000):

        options = " %s" % input_bam
        options += " %s" % output_prefix
        options += " -m %i" % max_memory

        self.execute(options, "samtools sort")




#!/usr/bin/env python

__author__ = 'mahajrod'

import os
from RouToolPa.Collections.General import IdList
from RouToolPa.Tools.Abstract import Tool
from RouToolPa.Routines import VCFRoutines, FileRoutines


class HaplotypeCaller4(Tool):
    def __init__(self,  max_threads=4,  max_memory=None, timelog=None):
        Tool.__init__(self,
                      "gatk HaplotypeCaller",
                      max_threads=max_threads, max_memory=max_memory, timelog=timelog)

    @staticmethod
    def parse_options_for_parallel_run(reference, alignment, output_mode=None,
                                       stand_call_conf=30, gvcf_mode=False):

        options = " -R %s" % reference

        if isinstance(alignment, str):
            options += " -I %s" % alignment
        else:
            for alignment_file in alignment:
                options += " -I %s" % alignment_file

        options += " --output_mode %s" % output_mode if output_mode else ""
        #options += " -stand_emit_conf %i" % stand_emit_conf
        options += " --standard-min-confidence-threshold-for-calling %i" % stand_call_conf
        options += " --emit-ref-confidence GVCF" if gvcf_mode else ""

        return options

    def parse_options(self, reference, alignment, output, output_mode=None,
                      stand_call_conf=30, gvcf_mode=False, include_region_id_file=None, exclude_region_id_file=None):

        #options = " -nct %i" % self.threads
        options = " -R %s" % reference
        if isinstance(alignment, str):
            options += " -I %s" % alignment
        else:
            for alignment_file in alignment:
                options += " -I %s" % alignment_file
        options += " --output_mode %s" % output_mode if output_mode else ""
        #options += " -stand_emit_conf %i" % stand_emit_conf
        options += " --standard-min-confidence-threshold-for-calling %i" % stand_call_conf
        options += " --emit-ref-confidence GVCF" if gvcf_mode else ""
        options += " -L %s" % include_region_id_file if include_region_id_file else ""
        options += " -XL %s" % exclude_region_id_file if exclude_region_id_file else ""

        options += " -O %s" % output

        return options

    def call(self, reference, alignment, output, output_mode=None,
             stand_call_conf=30, include_region_id_file=None, exclude_region_id_file=None):
        """
            gatk HaplotypeCaller \
              -R ${fasta} \
              -I ${bam%bam}realigned.bam \
              --output_mode EMIT_VARIANTS_ONLY \
              -stand_call_conf 30 \
              -o ${bam%bam}raw.vcf
        """
        options = self.parse_options(reference, alignment, output,
                                     output_mode=output_mode,
                                     stand_call_conf=stand_call_conf, gvcf_mode=False,
                                     include_region_id_file=include_region_id_file,
                                     exclude_region_id_file=exclude_region_id_file)

        self.execute(options,
                     cmd=("gatk --java-options -Xmx%s HaplotypeCaller" % self.max_memory) if self.max_memory else None)

    def gvcf_call(self, reference, alignment, output,
                  stand_call_conf=30, include_region_id_file=None, exclude_region_id_file=None):
        """
        gatk HaplotypeCaller \
         -R reference.fasta
         -I sample1.bam \
         --emit-ref-confidence GVCF \
         [--dbsnp dbSNP.vcf] \
         [-L targets.interval_list] \
         -o output.raw.snps.indels.g.vcf
        """
        options = self.parse_options(reference, alignment, output,
                                     stand_call_conf=stand_call_conf, gvcf_mode=True,
                                     include_region_id_file=include_region_id_file,
                                     exclude_region_id_file=exclude_region_id_file)

        self.execute(options,
                     cmd=("gatk --java-options -Xmx%s HaplotypeCaller" % self.max_memory) if self.max_memory else None)

    def parallel_call(self, reference, alignment, output_dir, output_prefix,
                      stand_call_conf=30, max_region_length=1000000, max_seqs_per_region=100,
                      length_dict=None, parsing_mode="parse", region_list=None,
                      region_file_format='simple',
                      remove_intermediate_files=False,
                      cpus_per_task=1,
                      handling_mode="local",
                      job_name=None,
                      log_prefix=None,
                      error_log_prefix=None,
                      max_running_jobs=None,
                      max_running_time=None,
                      max_memmory_per_cpu=None,
                      modules_list=None,
                      environment_variables_dict=None,
                      black_list_scaffold_id_file=None,
                      gvcf_mode=False):

        splited_dir = "%s/splited/" % output_dir
        regions_dir = "%s/regions/" % output_dir

        from RouToolPa.Tools.GATK4 import SortVcf4
        sequence_dict = reference[:-5] + "dict"
        SortVcf4.max_memory = self.max_memory
        SortVcf4.path = self.path

        for directory in output_dir, splited_dir:
            self.safe_mkdir(directory)

        if black_list_scaffold_id_file:
            if isinstance(black_list_scaffold_id_file, str):
                black_scaffolds_list = IdList(filename=black_list_scaffold_id_file)
            else:
                black_scaffolds_list = black_list_scaffold_id_file
        else:
            black_scaffolds_list = []
        region_list, \
            scaffold_to_region_correspondence_dict = self.prepare_region_list_by_length(max_length=max_region_length,
                                                                                        max_seq_number=max_seqs_per_region,
                                                                                        length_dict=length_dict,
                                                                                        reference=None if length_dict is not None else reference,
                                                                                        parsing_mode=parsing_mode,
                                                                                        output_dir=regions_dir,
                                                                                        black_list_scaffolds=black_scaffolds_list,
                                                                                        region_file_format=region_file_format if handling_mode != "slurm" else 'GATK') if region_list is None else region_list

        options = self.parse_options_for_parallel_run(reference, alignment,

                                                      stand_call_conf=stand_call_conf,
                                                      gvcf_mode=gvcf_mode)
        #options += " -nct 1"
        options_list = []

        output_index = 1

        output_file_list = []

        output_extension = "g.vcf" if gvcf_mode else "vcf"

        if handling_mode == 'local':
            for regions in region_list:

                output_file = "%s/%s_%i.%s" % (splited_dir, output_prefix, output_index, output_extension)
                region_options = " -O %s" % output_file
                output_file_list.append(output_file)
                #for region in regions:
                #    region_options += " -L %s:%i-%i" % (region[0], region[1], region[2])

                for region in regions:
                    if isinstance(region, str):
                        region_options += " -L %s" % region
                    elif len(region) == 1:
                        region_options += " -L %s" % region[0]
                    elif len(region) == 3:
                        region_options += " -L %s:%i-%i" % (region[0], region[1], region[2])

                options_list.append(options + region_options)
                output_index += 1
            print("Variant calling....")
            self.parallel_execute(options_list,
                                  cmd=("gatk --java-options -Xmx%s HaplotypeCaller" % self.max_memory) if self.max_memory else None)
            unsorted_combined_vcf = "%s/%s.unsorted.%s" % (output_dir, output_prefix, output_extension)
            sorted_combined_vcf = "%s/%s.%s" % (output_dir, output_prefix, output_extension)
            print("Combining variants...")
            VCFRoutines.combine_same_samples_vcfs(unsorted_combined_vcf,
                                                  vcf_list=output_file_list,
                                                  order_vcf_files=True,
                                                  close_fd_after=False,
                                                  extension_list=[".vcf",])
            print("Sorting...")
            SortVcf4.sort_vcf(unsorted_combined_vcf, sorted_combined_vcf, sequence_dict)

        elif handling_mode == 'slurm':
            number_of_regions = len(region_list)
            region_file = "%s/splited/region_${SLURM_ARRAY_TASK_ID}.list" % regions_dir
            output_file = "%s/%s_${SLURM_ARRAY_TASK_ID}.%s" % (splited_dir, output_prefix, output_extension)
            options += " -O %s" % output_file
            options += " -L %s" % region_file

            slurm_cmd = "gatk --java-options -Xmx%s HaplotypeCaller" % self.max_memory if self.max_memory else "gatk HaplotypeCaller"
            slurm_cmd += " %s" % options

            last_job_id = self.slurm_run_job(job_name,
                                             log_prefix,
                                             slurm_cmd,
                                             error_log_prefix,
                                             "%s%s.slurm" % (output_dir, output_prefix),
                                             task_index_list=None,
                                             start_task_index=1,
                                             end_task_index=number_of_regions,
                                             max_running_jobs=max_running_jobs,
                                             max_running_time=max_running_time,
                                             cpus_per_task=cpus_per_task,
                                             max_memmory_per_cpu=max_memmory_per_cpu,
                                             modules_list=modules_list,
                                             environment_variables_dict=environment_variables_dict)

            print("Submitted job  %s" % last_job_id)

            #VCFRoutines.combine_same_samples_vcfs(output_file_list,
            #                                      output,
            #                                      close_fd_after=False,
            #                                      extension_list=gvcf_extension_list)
        else:
            print("ERROR!!! Unrecognized handling mode!")



#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'

import os
from RouToolPa.Tools.Abstract import Tool


class Trimmer(Tool):
    def __init__(self, path="", max_threads=1):
        Tool.__init__(self, "V2_trim.exe", path=path, max_threads=max_threads)

    def parse_options(self, reads_prefix, kmer_file, output_prefix, write_fasta=False, input_format="fastq"):

        options = " %s" % reads_prefix
        options += " %s" % output_prefix
        options += " %i" % self.threads
        options += " %i" % (1 if write_fasta else 0)
        options += " %s" % input_format
        options += " %s" % kmer_file
        #options += " > %s.stats 2>&1" % output_prefix

        return options

    def trim_adapters(self, reads_prefix, adapter_kmer_file, output_prefix, write_fasta=False, input_format="fastq"):

        options = self.parse_options(reads_prefix, adapter_kmer_file, output_prefix,
                                     write_fasta=write_fasta, input_format=input_format)

        self.execute(options=options)

    def mass_trim(self, sample_dir, output_dir, adapter_kmer_file, sample_list=None,
                  write_fasta=False, input_format="fastq"):

        samples = self.get_sample_list(sample_dir, sample_list)

        self.safe_mkdir(output_dir)

        for sample in samples:
            sample_read_prefix = "%s/%s/%s" % (sample_dir, sample, sample)
            out_sample_dir = "%s/%s/" % (output_dir, sample)
            out_sample_prefix = "%s/%s" % (out_sample_dir, sample)
            self.safe_mkdir(out_sample_dir)

            self.trim_adapters(sample_read_prefix, adapter_kmer_file, out_sample_prefix,
                               write_fasta=write_fasta, input_format=input_format)


if __name__ == "__main__":
    pass





#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'

from RouToolPa.Tools.Abstract import Tool
from RouToolPa.Collections.General import SynDict, IdList


class Emapper(Tool):
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "emapper.py", path=path, max_threads=max_threads)

    def parse_options(self, input, database, output_prefix, db_type=None, query_type=None, target_orthologs=None,
                      resume=None, report_orthologs=True, override_last_run=None, no_comments_in_output=None,
                      no_refine=None, no_annotation=None, no_search=None, usemem=True):

        options = " -i %s" % input
        options += " --cpu %i" % self.threads
        options += " --usemem" if usemem else ""
        options += " --temp_dir %s" % self.tmp_dir if self.tmp_dir else ""
        options += " --database %s" % database
        options += " --dbtype %s" % db_type if db_type else ""
        options += " --qtype %s" % query_type if query_type else ""
        options += " --target_orthologs %s" % target_orthologs if target_orthologs else ""

        options += " -o %s" % output_prefix if output_prefix else ""

        options += " --resume" if resume else ""
        options += " --override" if override_last_run else ""
        options += " --no_refine" if no_refine else ""
        options += " --no_annot" if no_annotation else ""
        options += " --no_search" if no_search else ""
        options += " --no_file_comments" if no_comments_in_output else ""
        options += " --report_orthologs" if report_orthologs else ""

        return options

    def assign_orthologs(self, input, database, output_prefix, db_type=None, query_type=None, target_orthologs=None,
                         resume=None, report_orthologs=True, override_last_run=None, no_comments_in_output=None,
                         no_refine=None, no_annotation=None, no_search=None, usemem=True,
                         eggnogdb_prefix=None, species_name=None, label_separator="."):

        options = self.parse_options(input, database, output_prefix,
                                     db_type=db_type,
                                     query_type=query_type,
                                     target_orthologs=target_orthologs,
                                     resume=resume,
                                     report_orthologs=report_orthologs,
                                     override_last_run=override_last_run,
                                     no_comments_in_output=no_comments_in_output,
                                     no_refine=no_refine,
                                     no_annotation=no_annotation,
                                     no_search=no_search,
                                     usemem=usemem)

        self.execute(options)

        emapper_annotation_file = "%s.emapper.annotations" % output_prefix

        self.convert_emapper_annotation_file(emapper_annotation_file,
                                             output_prefix,
                                             eggnogdb_prefix=eggnogdb_prefix,
                                             species_name=species_name,
                                             label_separator=label_separator)

    @staticmethod
    def convert_emapper_annotation_file_to_fam(emapper_annotation_file, output_fam, eggnogdb_prefix=None,
                                               species_name=None, label_separator="."):
        fam_dict = SynDict()
        with open(emapper_annotation_file, "r") as annotations_fd:
            for line in annotations_fd:
                if line[0] == "#":
                    continue
                line_list = line.split("\t")

                fam_id = line_list[10].split("|")[0]
                if not(eggnogdb_prefix is None):
                    fam_id = eggnogdb_prefix + fam_id

                gene_id = "%s%s%s" % (species_name, label_separator, line_list[0]) if species_name else line_list[0]

                if fam_id in fam_dict:
                    fam_dict[fam_id].append(gene_id)
                else:
                    fam_dict[fam_id] = [gene_id]

        fam_dict.write(filename=output_fam, splited_values=True)

    @staticmethod
    def extract_emapper_annotations_by_protein_ids(emapper_annotation_file, protein_id_file, output_annotations):
        protein_ids = IdList(filename=protein_id_file)
        with open(emapper_annotation_file, "r") as ann_fd:
            with open(output_annotations, "w") as out_fd:
                for line in ann_fd:
                    if line[0] == "#":
                        out_fd.write(line)
                        continue
                    if line.split("\t")[0] in protein_ids:
                        out_fd.write(line)
    @staticmethod
    def extract_GO_terms_from_emapper_annotation_file(emapper_annotation_file, output_file):
        GO_terms_dict = SynDict(filename=emapper_annotation_file, key_index=0, value_index=5,
                                split_values=True, values_separator=",", comments_prefix="#",
                                separator="\t")
        GO_terms_dict.header = "#protein_id\tGO_terms"
        GO_terms_dict.write(output_file, header=True, splited_values=True)

        return GO_terms_dict

    @staticmethod
    def extract_predicted_gene_names_from_emapper_annotation_file(emapper_annotation_file, output_file):
        extract_predicted_gene_names_dict = SynDict(filename=emapper_annotation_file, key_index=0, value_index=4,
                                                    split_values=True, values_separator=",", comments_prefix="#",
                                                    separator="\t")
        extract_predicted_gene_names_dict.header = "#protein_id\tpredicted_gene_name"
        extract_predicted_gene_names_dict.write(output_file, header=True, splited_values=True)

        return extract_predicted_gene_names_dict

    def convert_emapper_annotation_file(self, emapper_annotation_file, output_prefix, eggnogdb_prefix=None,
                                         species_name=None, label_separator="."):
        fam_file = "%s.fam" % output_prefix
        GO_terms_file = "%s.GO" % output_prefix
        predicted_gene_names_file = "%s.predicted_gene_names" % output_prefix

        self.convert_emapper_annotation_file_to_fam(emapper_annotation_file, fam_file,
                                                    eggnogdb_prefix=eggnogdb_prefix,
                                                    species_name=species_name,
                                                    label_separator=label_separator)
        self.extract_GO_terms_from_emapper_annotation_file(emapper_annotation_file, GO_terms_file)

        self. extract_predicted_gene_names_from_emapper_annotation_file(emapper_annotation_file,
                                                                        predicted_gene_names_file)

if __name__ == "__main__":
    pass



#!/usr/bin/env python
__author__ = 'mahajrod'

import re
import os
import sys
import bz2
import gzip
import shutil
from collections import Iterable, OrderedDict

if sys.version_info[0] == 3:
    from io import TextIOWrapper as file

from RouToolPa.Collections.General import IdSet,  IdList, SynDict


class FileRoutines:
    def __init__(self):
        self.filetypes_dict = {"fasta": [".fa", ".fasta", ".fa", ".pep", ".cds"],
                               "fastq": [".fastq", ".fq"],
                               "genbank": [".gb", ".genbank"],
                               "newick": [".nwk"],
                               "gz": [".gz"],
                               "bzip": [".bz2"],
                               "bam": [".bam"],
                               "sam": [".sam"],
                               "cram": [".cram"]}

    @staticmethod
    def metaopen(filename, flags):
        if isinstance(filename, file):
            return filename
        elif filename[-3:] == ".gz":
            return gzip.open(filename, flags)
        elif filename[-4:] == ".bz2":
            return bz2.open(filename, flags)
        else:
            return open(filename, flags)

    @staticmethod
    def add_external_extraction_to_filename(filename):
        if filename[-3:] == ".gz":
            return "<(gunzip -c %s)" % filename
        elif filename[-4:] == ".bz2":
            return "<(bunzip2 -c %s)" % filename
        else:
            return filename

    @staticmethod
    def add_external_extraction_to_filelist(filelist):
        new_filelist = []

        for filename in filelist:
            if filename[-3:] == ".gz":
                new_filelist.append("<(gunzip -c %s)" % filename)
            elif filename[-4:] == ".bz2":
                new_filelist.append("<(bunzip2 -c %s)" % filename)
            else:
                new_filelist.append(filename)

        return new_filelist

    @staticmethod
    def safe_mkdir(dirname, 
                   description_filename=None, description_text=None,
                   readme_filename=None, readme_text=None):
        try:
            os.mkdir(dirname)
        except OSError:
            pass
        
        if not(description_filename is None):
            description_filename = "%s/%s" % (dirname, description_filename)
    
            if not os.path.isfile(description_filename):
                with open(description_filename, "w") as descr_fd:
                    if not (description_text is None):
                        descr_fd.write(description_text)
                        
        if not(readme_filename is None):
            readme_filename = "%s/%s" % (dirname, readme_filename)
    
            if not os.path.isfile(readme_filename):
                with open(readme_filename, "w") as descr_fd:
                    if not (readme_text is None):
                        descr_fd.write(readme_text)

    def recursive_mkdir(self, dir_dict=None, out_dir=None,
                        description_filename=None, description_text=None,
                        readme_filename=None, readme_text=None):
        if not(out_dir is None):
            self.safe_mkdir(out_dir)

        if dir_dict:
            for directory in dir_dict:
                dirname = directory if out_dir is None else "%s/%s" % (out_dir, directory)
                self.safe_mkdir(dirname,
                                description_filename=description_filename,
                                description_text=description_text,
                                readme_filename=readme_filename,
                                readme_text=readme_text)

                if isinstance(dir_dict[directory], dict):
                    self.recursive_mkdir(dir_dict[directory],
                                         out_dir=dirname,
                                         description_filename=description_filename,
                                         description_text=description_text,
                                         readme_filename=readme_filename,
                                         readme_text=readme_text)

    def detect_filetype_by_extension(self, filename, filetypes_dict=None):
        filetypes = filetypes_dict if filetypes_dict else self.filetypes_dict
        directory, prefix, extension = self.split_filename(filename)
        for filetype in filetypes:
            if extension in filetypes[filetype]:
                return filetype
        return None

    @staticmethod
    def check_path(path_to_check):
        #print (path_to_check)
        #returns path with / at end or blank path
        if path_to_check != "":
            if path_to_check[-1] != "/":
                return path_to_check + "/"
        return path_to_check

    @staticmethod
    def check_id(id, white_list=(), black_list=()):
        if white_list and black_list:
            if (id in white_list) and (id not in black_list):
                return True
            return False
        elif white_list:
            return True if (id in white_list) else False
        elif black_list:
            return False if (id in black_list) else True
        else:
            return True

    @staticmethod
    def check_dir_path(path_to_check):
        #print (path_to_check)
        #returns path with / at end or blank path
        if path_to_check != "":
            if path_to_check[-1] != "/":
                return path_to_check + "/"
        return path_to_check

    @staticmethod
    def split_filename(filepath):
        directory, basename = os.path.split(filepath)
        if directory:
            if directory[-1] != "/":
                directory += "/"
        prefix, extension = os.path.splitext(basename)
        return directory, prefix, extension if filepath else None

    @staticmethod
    def get_basename(filepath):
        return os.path.split(filepath)[-1] if filepath else None

    def make_list_of_path_to_files(self, list_of_dirs_and_files, expression=None, recursive=False,
                                   return_absolute_paths=True):
        file_list = []
        for entry in [list_of_dirs_and_files] if isinstance(list_of_dirs_and_files, str) else list_of_dirs_and_files:
            if os.path.isdir(entry):
                files_in_dir = ["%s%s" % (self.check_path(entry), filename)
                                for filename in sorted(filter(expression, os.listdir(entry))
                                                       if expression else os.listdir(entry))]
                if recursive:
                    for filename in files_in_dir:
                        if os.path.isdir(filename):
                            file_list += self.make_list_of_path_to_files([filename],
                                                                         expression=expression,
                                                                         recursive=recursive)
                        else:
                            file_list.append(filename)
                else:
                    file_list += files_in_dir
            elif os.path.exists(entry):
                if expression:
                    if expression(os.path.abspath(entry)):
                        file_list.append(os.path.abspath(entry))
                        """
                        print ("BBBBBBBBBBBBBB")
                        print (entry)
                    else:
                        print ("AAAAAAAAAAA")
                        print (entry)
                        """
                else:
                    file_list.append(os.path.abspath(entry))
            else:
                print("%s does not exist" % entry)
        # direct conversion to list was added for compatibility with python3
        # in which map function returns map object instead of list
        return list(map(os.path.abspath, file_list)) if return_absolute_paths else file_list

    def make_list_of_path_to_files_from_string(self, input_string, file_separator=",",
                                               expression=None, recursive=False):
        return self.make_list_of_path_to_files(input_string.split(file_separator),
                                               expression=expression,
                                               recursive=recursive)

    @staticmethod
    def split_string_by_separator(string, separator):
        return string.split(separator)

    def split_string_by_comma(self, string):
        return self.split_string_by_separator(string, separator=",")

    def split_string_by_colon(self, string):
        return self.split_string_by_separator(string, separator=":")

    @staticmethod
    def check_extension(filename, extension_list=[]):
        if extension_list:
            for extension in extension_list:
                #print extension, filename[-len(extension):]
                if extension == filename[-len(extension):]:
                    return True
        else:
            #print "CCCCCCCCCCCCCCCCC"
            #print filename
            return True

        return False

    def make_list_of_path_to_files_by_extension(self, list_of_dirs_and_files, extension_list=[], recursive=False,
                                                return_absolute_paths=True):

        def check_extension(filename):
            return self.check_extension(filename, extension_list)

        return self.make_list_of_path_to_files(list_of_dirs_and_files, expression=check_extension,
                                               recursive=recursive, return_absolute_paths=return_absolute_paths)

    @staticmethod
    def read_synonyms_dict(filename, header=False, separator="\t",
                           split_values=False, values_separator=",", key_index=0, value_index=1):
        # reads synonyms from file
        synonyms_dict = OrderedDict()
        with open(filename, "r") as in_fd:
            if header:
                header_str = in_fd.readline().strip()
            for line in in_fd:
                tmp = line.strip().split(separator)
                #print line
                key, value = tmp[key_index], tmp[value_index]
                if split_values:
                    value = value.split(values_separator)
                synonyms_dict[key] = value
        return synonyms_dict

    @staticmethod
    def read_ids(filename, header=False, close_after_if_file_object=False):
        #reads ids from file or file object with one id per line
        id_list = []

        in_fd = filename if isinstance(filename, file) else open(filename, "r")

        if header:
            header_str = in_fd.readline().strip()
        for line in in_fd:
            id_list.append(line.strip())
        if (not isinstance(filename, file)) or close_after_if_file_object:
            in_fd.close()

        return id_list

    @staticmethod
    def read_tsv_as_rows_list(filename, header=False, header_prefix="#", separator="\t"):
        tsv_list = []
        with open(filename, "r") as tsv_fd:
            if header:
                header_list = tsv_fd.readline().strip()[len(header_prefix):].split(separator)
            for line in tsv_fd:
                tsv_list.append(line.strip().split(separator))
        return header_list, tsv_list if header else tsv_list

    @staticmethod
    def read_tsv_as_columns_dict(filename, header_prefix="#", separator="\t"):
        tsv_dict = OrderedDict()
        with open(filename, "r") as tsv_fd:
            header_list = tsv_fd.readline().strip()[len(header_prefix):].split(separator)
            number_of_columns = len(header_list)
            for column_name in header_list:
                tsv_dict[column_name] = []
            for line in tsv_fd:
                tmp_line = line.strip().split(separator)
                for i in range(0, number_of_columns):
                    tsv_dict[header_list[i]].append(tmp_line[i])
        return tsv_dict

    @staticmethod
    def tsv_split_by_column(tsv_file, column_number, separator="\t", header=False, outfile_prefix=None):
        # column number should start from 0
        header_string = None
        splited_name = tsv_file.split(".")
        extension = splited_name[-1] if len(splited_name) > 1 else ""
        out_prefix = outfile_prefix if outfile_prefix is not None \
            else ".".join(splited_name[:-1]) if len(splited_name) > 1 else splited_name[0]
        out_fd_dict = {}

        with open(tsv_file, "r") as in_fd:
            if header:
                header_string = in_fd.readline()
            for line in in_fd:
                line_str = line.strip().split(separator)
                if line_str[column_number] not in out_fd_dict:
                    print (line_str[column_number])
                    suffix = line_str[column_number].replace(" ", "_")
                    out_name = "%s_%s.%s" % (out_prefix, suffix, extension)
                    out_fd_dict[line_str[column_number]] = open(out_name, "w")
                    if header:
                        out_fd_dict[line_str[column_number]].write(header_string)
                out_fd_dict[line_str[column_number]].write(line)

        for entry in out_fd_dict:
            out_fd_dict[entry].close()

    def tsv_extract_by_column_value(self, tsv_file, column_number, column_value,
                                    output_file, comments_prefix="#", header=False, separator="\t"):
        # column number should start from 0
        # column_value should be string or list of strings

        comments_prefix_len = len(comments_prefix)

        column_value_list = column_value if isinstance(column_value, Iterable) else []
        with self.metaopen(tsv_file, "r") as in_fd, self.metaopen(output_file, "w") as out_fd:
            if header:
                out_fd.write(in_fd.readline())
            for line in in_fd:
                if line[:comments_prefix_len] == comments_prefix:
                    out_fd.write(line)
                line_str = line.strip().split(separator)
                if line_str[column_number] in column_value_list:
                    out_fd.write(line)
        out_fd.close()

    @staticmethod
    def tsv_remove_by_column_value(tsv_file, column_number, column_value, separator="\t",
                                   header=False, outfile_prefix=None):
        # column number should start from 0
        # column_value should be string or list of strings

        splited_name = tsv_file.split(".")
        extension = splited_name[-1] if len(splited_name) > 1 else ""
        out_prefix = outfile_prefix if outfile_prefix is not None \
            else ".".join(splited_name[:-1]) if len(splited_name) > 1 else splited_name[0]
        out_fd = open("%s.%s" % (out_prefix, extension), "w")

        column_value_list = column_value if isinstance(column_value, Iterable) else []
        with open(tsv_file, "r") as in_fd:
            if header:
                out_fd.write(in_fd.readline())
            for line in in_fd:
                line_str = line.strip().split(separator)
                if line_str[column_number] not in column_value_list:
                    out_fd.write(line)
        out_fd.close()

    @staticmethod
    def make_lists_forward_and_reverse_files(sample_dir, filename_fragment_to_mark_se_reads=".se.", input_is_se=False):
        file_list = sorted(os.listdir(sample_dir))
        filtered_filelist = []
        filetypes = set()
        for entry in file_list:
            if entry[-3:] == ".fq" or entry[-6:] == ".fastq":
                filetypes.add("fq")
            elif entry[-6:] == ".fq.gz" or entry[-9:] == ".fastq.gz":
                filetypes.add("fq.gz")
            elif entry[-7:] == ".fq.bz2" or entry[-10:] == ".fastq.bz2":
                filetypes.add("fq.bz2")
            else:
                continue
            filtered_filelist.append("%s/%s" % (sample_dir, entry))

        if input_is_se:
            return filetypes, [], [], filtered_filelist

        single_end_filelist = []
        paired_end_filelist = []

        for entry in filtered_filelist:
            if filename_fragment_to_mark_se_reads in entry:
                single_end_filelist.append(entry)
            else:
                paired_end_filelist.append(entry)

        forward_files = paired_end_filelist[::2]
        reverse_files = paired_end_filelist[1:][::2]

        if len(filetypes) > 1:
            print("WARNING: mix of archives of different types and/or uncompressed files")

        return filetypes, forward_files, reverse_files, single_end_filelist

    @staticmethod
    def get_sample_list(samples_directory):
        samples = sorted(os.listdir(samples_directory))
        sample_list = []
        for sample in samples:
            if os.path.isdir("%s/%s" % (samples_directory, sample)):
                sample_list.append(sample)
        return sample_list

    @staticmethod
    def extract_ids_from_file(input_file, output_file=None, header=False, column_separator="\t",
                              comments_prefix="#", column_number=None):
        id_list = IdList()
        id_list.read(input_file, column_separator=column_separator, comments_prefix=comments_prefix,
                     column_number=column_number, header=header)
        if output_file:
            id_list.write(output_file, header=header)
        return id_list

    @staticmethod
    def intersect_ids_from_files(files_with_ids_from_group_a, files_with_ids_from_group_b,
                                 result_file=None, mode="common", case_insensitive=False):
        a = IdSet()
        b = IdSet()

        if mode == "common":
            expression = lambda a, b: a & b
        elif mode == "only_a":
            expression = lambda a, b: a - b
        elif mode == "only_b":
            expression = lambda a, b: b - a
        elif mode == "not_common":
            expression = lambda a, b: a ^ b
        elif mode == "combine":
            expression = lambda a, b: a | b

        #print(files_with_ids_from_group_a)
        for filename in [files_with_ids_from_group_a] if isinstance(files_with_ids_from_group_a, str) else files_with_ids_from_group_a:
            id_set = IdSet()
            id_set.read(filename, comments_prefix="#", expression=(lambda s: s.upper()) if case_insensitive else None)
            a = a | id_set

        for filename in [files_with_ids_from_group_b] if isinstance(files_with_ids_from_group_b, str) else files_with_ids_from_group_b:
            id_set = IdSet()
            id_set.read(filename, comments_prefix="#", expression=(lambda s: s.upper()) if case_insensitive else None)
            b = b | id_set
        result_fd = open(result_file, "w") if result_file else sys.stdout
        if mode != "count":
            final_set = IdSet(expression(a, b))
            final_set.write(result_fd)
        else:
            result_fd.write("Group_A\t%i\nGroup_B\t%i\nCommon\t%i\nOnly_group_A\t%i\nOnly_group_B\t%i\nNot_common\t%i\nAll\t%i\n" %
                            (len(a), len(b), len(a & b), len(a - b), len(b - a), len(a ^ b), len(a | b)))

    @staticmethod
    def intersect_ids(list_of_group_a, list_of_group_b, mode="common"):
        # possible modes: common, only_a, only_b, not_common,  combine, count
        a = IdSet()
        b = IdSet()

        if mode == "common":
            expression = lambda a, b: a & b
        elif mode == "only_a":
            expression = lambda a, b: a - b
        elif mode == "only_b":
            expression = lambda a, b: b - a
        elif mode == "not_common":
            expression = lambda a, b: a ^ b
        elif mode == "combine":
            expression = lambda a, b: a | b

        for id_list in list_of_group_a:
            a = a | IdSet(id_list)

        for id_list in list_of_group_b:
            b = b | IdSet(id_list)

        if mode != "count":
            return IdSet(expression(a, b))
        else:
            return len(a), len(b), len(a & b), len(a - b), len(b - a), len(a ^ b), len(a | b)

    @staticmethod
    def split_by_column(input_file, column_number, separator="\t", header=False, outfile_prefix=None,
                        use_column_value_as_prefix=False, sorted_input=False):
        # column number should start from 0
        # use sorted input to reduce number of simalteniously open files
        header_string = None
        splited_name = input_file.split(".")
        extension = splited_name[-1] if len(splited_name) > 1 else ""
        out_prefix = outfile_prefix if outfile_prefix is not None \
            else ".".join(splited_name[:-1]) if len(splited_name) > 1 else splited_name[0]
        out_fd_dict = {}
        previous_value = None
        with open(input_file, "r") as in_fd:
            if header:
                header_string = in_fd.readline()
            for line in in_fd:
                line_str = line.strip().split(separator)
                if line_str[column_number] not in out_fd_dict:
                    print (line_str[column_number])
                    if previous_value and sorted_input:
                        out_fd_dict[previous_value].close()
                    suffix = line_str[column_number].replace(" ", "_")
                    out_name = "%s.%s" % (suffix, extension) if use_column_value_as_prefix else "%s.%s.%s" % (out_prefix, suffix, extension)
                    out_fd_dict[line_str[column_number]] = open(out_name, "w")
                    if header:
                        out_fd_dict[line_str[column_number]].write(header_string)

                out_fd_dict[line_str[column_number]].write(line)
                previous_value = line_str[column_number]
        if sorted_input:
            out_fd_dict[previous_value].close()
        else:
            for entry in out_fd_dict:
                out_fd_dict[entry].close()

    @staticmethod
    def get_sample_list(sample_dir, sample_list=None):
        samples = []
        if sample_list:
            return [sample_list] if isinstance(sample_list, str) else sample_list
        else:
            dir_list = os.listdir(sample_dir)
            for directory in dir_list:
                if os.path.isdir("%s/%s" % (sample_dir, directory)):
                    samples.append(directory)

            return samples

    @staticmethod
    def replace_column_value_by_syn(input_file, syn_file, out_file, column=0, comment_prefix=None, separator="\t",
                                    syn_header=False, syn_separator="\t",
                                    syn_key_index=0, syn_value_index=1, syn_comment_prefix=None):
        syn_dict = SynDict(filename=syn_file, header=syn_header, separator=syn_separator, key_index=syn_key_index,
                           value_index=syn_value_index, comments_prefix=syn_comment_prefix)
        if comment_prefix:
            comment_prefix_len = len(comment_prefix)
        line_number = 0
        replaced = 0
        not_replaced = 0
        with open(input_file, "r") as in_fd:
            with open(out_file, "w") as out_fd:
                for line in in_fd:
                    line_number += 1
                    if comment_prefix:
                        if line[0:comment_prefix_len] == comment_prefix:
                            out_fd.write(line)
                            continue
                    line_list = line.strip("\n").split(separator)
                    if len(line_list) < column + 1:
                        sys.stderr.write("WARNING!!! Line %i doesn't have column %i\n" % (line_number, column))
                    if line_list[column] in syn_dict:
                        replaced += 1
                        line_list[column] = syn_dict[line_list[column]]
                    else:
                        not_replaced += 1

                    out_fd.write(separator.join(line_list))
                    out_fd.write("\n")

        sys.stderr.write("Replaced: %i\nNot replaced: %i\n" % (replaced, not_replaced))

    @staticmethod
    def combine_syn_dicts(list_of_syn_dict):
        combined_dict = SynDict()
        for syn_dict in list_of_syn_dict:
            for key in syn_dict:
                if key in combined_dict:
                    combined_dict[key] += syn_dict[key]
                else:
                    combined_dict[key] = syn_dict[key]

        return combined_dict

    def combine_syn_dicts_from_file(self, list_of_syndict_files, output, key_index=0, value_index=1, separator="\t",
                                    values_separator=",", header=False):
        list_of_syn_dicts = []
        for filename in list_of_syndict_files:
            list_of_syn_dicts.append(SynDict(filename=filename, key_index=key_index, value_index=value_index,
                                             separator=separator, values_separator=values_separator, split_values=True,
                                             allow_repeats_of_key=True, header=header))

        merged_dict = self.combine_syn_dicts(list_of_syn_dicts)

        merged_dict.write(output, splited_values=True)

    def add_add_new_column_by_key_column(self, table_file, syn_dict_file, key_column,  output_file, new_column_name=None,
                                         separator='\t', absent_value="."):
        column_syn_dict = SynDict(filename=syn_dict_file, allow_repeats_of_key=True, values_separator="@")
        with open(table_file, "r") as in_fd, open(output_file, "w") as out_fd:
            if new_column_name:
                header_line = in_fd.readline().strip() + "\t%s\n" % new_column_name
                out_fd.write(header_line)
                for line in in_fd:
                    line_list = line.strip().split(separator)
                    if line_list[key_column] in column_syn_dict:
                        print (line_list[key_column])
                        print (column_syn_dict[line_list[key_column]])
                    line_list.append(absent_value if line_list[key_column] not in column_syn_dict else "|".join(column_syn_dict[line_list[key_column]]))
                    out_fd.write(separator.join(line_list) + "\n")

    @staticmethod
    def label_column_in_file(input_file, label, column_index, output_file, column_separator="\t",
                             label_position="first", label_separator="@",
                             comments_prefix="#"):
        with open(input_file, "r") as in_fd:
            with open(output_file, "w") as out_fd:
                for line in in_fd:
                    if line[0] == comments_prefix:
                        out_fd.write(line)
                        continue

                    line_list = line.strip("\n").split(column_separator)
                    if label_position == "first":
                        line_list[column_index] = "%s%s%s" % (label, label_separator, line_list[column_index])
                    elif label_position == "last":
                        line_list[column_index] = "%s%s%s" % (line_list[column_index], label_separator, label)
                    else:
                        raise ValueError("ERROR!!!Unrecognized label position")

                    out_fd.write(column_separator.join(line_list) + "\n")

    def file_line_as_list_generator(self, input_file, comments_prefix="#", separator="\t"):
        comments_prefix_len = len(comments_prefix)

        with self.metaopen(input_file, "r") as in_fd:
            for line in in_fd:
                if line[:comments_prefix_len] == comments_prefix:
                    continue
                yield line.strip().split(separator)

    def file_line_generator(self, input_file, comments_prefix="#"):
        comments_prefix_len = len(comments_prefix)

        with self.metaopen(input_file, "r") as in_fd:
            for line in in_fd:
                if line[:comments_prefix_len] == comments_prefix:
                    continue
                yield line.strip()

    @staticmethod
    def combine_files_with_header(file_list, output_file, header_prefix="#", sorting_options=None):

        #list_of_files = self.make_list_of_path_to_files(file_list)
        unsorted_file = "%s.unsorted.tmp" % output_file

        string = "cat %s > %s" % (file_list[0], unsorted_file if sorting_options else output_file)
        os.system(string)

        for filename in file_list[1:]:
            string = "sed -n '/^[^%s]/p' %s >> %s" % (header_prefix,
                                                      filename,
                                                      unsorted_file if sorting_options else output_file)
            print(string)
            os.system(string)

        if sorting_options:
            sorting_string = "(sed '/^[^#]/Q' %s; sort %s %s) > %s" % (unsorted_file,
                                                                       sorting_options,
                                                                       unsorted_file,
                                                                       output_file)
            print(sorting_string)

            os.system(sorting_string)

    def combine_chunks_with_header(self, chunks_dir, chunks_prefix, output_file,
                                   starting_chunk=None, end_chunk=None, chunk_number_list=None,
                                   chunks_suffix=None, header_prefix="#", sorting_options=None,
                                   separator="_"):

        if chunk_number_list:
            file_list = []
            for chunk_n in chunk_number_list:
                file_list.append("%s/%s%s%s%s" % (chunks_dir, chunks_prefix, separator,
                                                  str(chunk_n), chunks_suffix if chunks_suffix else ""))

        elif starting_chunk or end_chunk:
            if starting_chunk and end_chunk:
                file_list = []
                for chunk_n in range(starting_chunk, end_chunk + 1):
                    file_list.append("%s/%s%s%s%s" % (chunks_dir, chunks_prefix, separator,
                                                      str(chunk_n), chunks_suffix if chunks_suffix else ""))
            else:
                raise ValueError("Either starting or end chunks was not set")
        else:
            file_list = self.make_list_of_path_to_files(chunks_dir)

        self.combine_files_with_header(file_list, output_file,
                                       header_prefix=header_prefix, sorting_options=sorting_options)

    @staticmethod
    def get_filtered_entry_list(entry_list,
                                entry_black_list=[],
                                sort_entries=False,
                                entry_ordered_list=None,
                                entry_white_list=[]):
        white_set = set(entry_white_list)
        black_set = set(entry_black_list)
        entry_set = set(entry_list)

        if white_set:
            entry_set = entry_set & white_set
        if black_set:
            entry_set = entry_set - black_set

        filtered_entry_list = list(entry_set)
        if sort_entries:
            filtered_entry_list.sort()

        final_entry_list = []

        if entry_ordered_list:
            for entry in entry_ordered_list:
                if entry in filtered_entry_list:
                    final_entry_list.append(entry)
                    filtered_entry_list.remove(entry)
                else:
                    print("WARNING!!!Entry(%s) from order list is absent in list of entries!" % entry)
            return final_entry_list + filtered_entry_list
        else:
            return filtered_entry_list

    def merge_files_by_columns(self, file_list, column_index_list,  output_file, separator="\t", column_names_list=None,
                               comment_prefix="#", header=False):
        comment_prefix_len = len(comment_prefix)
        number_of_files = len(file_list)

        if len(column_index_list) != number_of_files:
            raise ValueError("ERROR!!! Column indexes either were not specified for some files "
                             "or specified too many of them")

        combined_header_list = []

        file_line_dict_list = []

        for file_index in range(0, number_of_files):
            file_line_dict_list.append(OrderedDict())

            with self.metaopen(file_list[file_index], "r") as file_fd:
                if header:
                    header_list = file_fd.readline().strip("\n").split()
                    for i in range(0, len(header_list)):
                        if i in column_index_list[file_index]:
                            continue
                        combined_header_list.append(header_list[i])

                for line in file_fd:
                    if line[0:comment_prefix_len] == comment_prefix:
                        continue
                    line_list = line.strip().split(separator)

                    line_combined_id = ""
                    for column_index in column_index_list[file_index]:
                        line_combined_id += line_list[column_index]
                    file_line_dict_list[-1][line_combined_id] = line_list

        file_line_dict_line_len = []
        for file_dict in file_line_dict_list:
            file_line_dict_line_len.append(len(file_dict[list(file_dict.keys())[0]]))

        with self.metaopen(output_file, "w") as out_fd:
            if header:
                out_fd.write(separator.join(column_names_list + combined_header_list) + "\n")

            for line_id in file_line_dict_list[0]:
                for j in range(1, len(file_line_dict_list)):
                    if line_id not in file_line_dict_list[j]:
                        break
                else:
                    output_line_list = []
                    for column_index in column_index_list[0]:
                        output_line_list.append(file_line_dict_list[0][line_id][column_index])

                    for file_dict_index in range(0, len(file_line_dict_list)):
                        for column_index in range(0, file_line_dict_line_len[file_dict_index]):
                            if column_index in column_index_list[file_dict_index]:
                                continue
                            output_line_list.append(file_line_dict_list[file_dict_index][line_id][column_index])

                    out_fd.write(separator.join(output_line_list) + "\n")

    def rename_ncbi_genome_files(self, genome_dir):
        species_list = os.listdir(genome_dir)

        assembly_suffix = "_genomic.fna"
        gff_suffix = "_genomic.gff"
        protein_suffix = "_protein.faa"
        rna_suffix = "_rna.fna"
        tab_suffix = ".txt"
        for species in species_list:
            species_path = "%s/%s" % (genome_dir, species)
            source_list = os.listdir(species_path)
            for source in source_list:
                species_source_path = "%s/%s" % (species_path, source)
                if os.path.isdir(species_source_path):
                    assembly_list = os.listdir(species_source_path)
                    for assembly in assembly_list:
                        species_source_assembly_path = "%s/%s" % (species_source_path, assembly)
                        if os.path.isdir(species_source_assembly_path):
                            os.system("gunzip %s/*" % species_source_assembly_path)
                            file_list = os.listdir(species_source_assembly_path)
                            for filename in file_list:
                                filename_path = "%s/%s" % (species_source_assembly_path, filename)
                                if os.path.isfile(filename_path):
                                    if assembly_suffix in filename:
                                        shutil.move(filename_path, "%s/%s.fasta" % (species_source_assembly_path, species))
                                    elif gff_suffix in filename:
                                        shutil.move(filename_path, "%s/%s.gff" % (species_source_assembly_path, species))
                                    elif protein_suffix in filename:
                                        shutil.move(filename_path, "%s/%s.pep" % (species_source_assembly_path, species))
                                    elif rna_suffix in filename:
                                        shutil.move(filename_path, "%s/%s.transcript" % (species_source_assembly_path, species))
                                    elif tab_suffix in filename:
                                        shutil.move(filename_path, "%s/%s.tab" % (species_source_assembly_path, species))



    @staticmethod
    def string2float(string):
        try:
            retval = float(string)
        except ValueError:
            retval = string
        return retval

    @staticmethod
    def string2int(string):
        return int(string) if string.isdigit() else string

    def natural_keys_int(self, string):
        """
        alist.sort(key=natural_keys) sorts in human order
        http://nedbatchelder.com/blog/200712/human_sorting.html
        (See Toothy's implementation in the comments)
        """

        return [self.string2int(c) for c in re.split('(\d+)', string)]

    def natural_keys_float(self, string):
        """
        alist.sort(key=natural_keys) sorts in human order
        http://nedbatchelder.com/blog/200712/human_sorting.html
        (See Toothy's implementation in the comments)
        float regex comes from https://stackoverflow.com/a/12643073/190597
        """
        return [self.string2float(c) for c in re.split(r'[+-]?([0-9]+(?:[.][0-9]*)?|[.][0-9]+)', string)]

    def get_column_value_set_from_file(self, input_file, column_number, output_file=None,
                                       separator="\t", comments_prefix="#", verbose=False):

        column_value_set = IdSet([line_list[column_number] for line_list in self.file_line_as_list_generator(input_file,
                                                                                                             separator=separator,
                                                                                                             comments_prefix=comments_prefix)])
        if output_file:
            column_value_set.write(output_file)

        if verbose:
            print("#Column %i (0-based) contains %i different values" % (column_number, len(column_value_set)))

        return column_value_set

    def count_column_values_from_file(self, input_file, column_number, output_file=None,
                                      separator="\t", comments_prefix="#", verbose=False):

        column_value_dict = SynDict()

        for line_list in self.file_line_as_list_generator(input_file,
                                                          separator=separator,
                                                          comments_prefix=comments_prefix):

            if line_list[column_number] in column_value_dict:
                column_value_dict[line_list[column_number]] += 1
            else:
                column_value_dict[line_list[column_number]] = 1

        if output_file:
            column_value_dict.write(output_file)

        if verbose:
            print("#Column %i (0-based) contains %i different values" % (column_number, len(column_value_set)))

        return column_value_dict

filetypes_dict = {"fasta": [".fa", ".fasta", ".fa", ".pep", ".cds"],
                  "fastq": [".fastq", ".fq"],
                  "genbank": [".gb", ".genbank"],
                  "newick": [".nwk"]}


def safe_mkdir(dirname):
    try:
        os.mkdir(dirname)
    except OSError:
        pass


def detect_filetype_by_extension(filename, filetypes_dict=filetypes_dict):
    directory, prefix, extension = split_filename(filename)
    for filetype in filetypes_dict:
        if extension in filetypes_dict[filetype]:
            return filetype
    return None


def check_path(path_to_check):
    #print (path_to_check)
    #returns path with / at end or blank path
    if path_to_check != "":
        if path_to_check[-1] != "/":
            return path_to_check + "/"
    return path_to_check


def split_filename(filepath):
    directory, basename = os.path.split(filepath)
    prefix, extension = os.path.splitext(basename)
    return directory, prefix, extension


def make_list_of_path_to_files(list_of_dirs_and_files, expression=None):

    paths_list = []
    for entry in list_of_dirs_and_files:
        #print entry
        if os.path.isdir(entry):
            files_in_dir = sorted(filter(expression, os.listdir(entry)) if expression else os.listdir(entry))
            for filename in files_in_dir:
                paths_list.append("%s%s" % (check_path(entry), filename))
        elif os.path.exists(entry):
            if expression:
                if expression(entry):
                    paths_list.append(entry)
            else:
                paths_list.append(entry)
        else:
            print("%s does not exist" % entry)

    return paths_list


def read_synonyms_dict(filename, header=False, separator="\t",
                       split_values=False, values_separator=",", key_index=0, value_index=1):
    # reads synonyms from file
    synonyms_dict = OrderedDict()
    with open(filename, "r") as in_fd:
        if header:
            header_str = in_fd.readline().strip()
        for line in in_fd:
            tmp = line.strip().split(separator)
            #print line
            key, value = tmp[key_index], tmp[value_index]
            if split_values:
                value = value.split(values_separator)
            synonyms_dict[key] = value
    return synonyms_dict


def read_ids(filename, header=False, close_after_if_file_object=False):
    #reads ids from file or file object with one id per line
    id_list = []

    in_fd = filename if isinstance(filename, file) else open(filename, "r")

    if header:
        header_str = in_fd.readline().strip()
    for line in in_fd:
        id_list.append(line.strip())
    if (not isinstance(filename, file)) or close_after_if_file_object:
        in_fd.close()

    return id_list


def read_tsv_as_rows_list(filename, header=False, header_prefix="#", separator="\t"):
    tsv_list = []
    with open(filename, "r") as tsv_fd:
        if header:
            header_list = tsv_fd.readline().strip()[len(header_prefix):].split(separator)
        for line in tsv_fd:
            tsv_list.append(line.strip().split(separator))
    return header_list, tsv_list if header else tsv_list


def read_tsv_as_columns_dict(filename, header_prefix="#", separator="\t"):
    tsv_dict = OrderedDict()
    with open(filename, "r") as tsv_fd:
        header_list = tsv_fd.readline().strip()[len(header_prefix):].split(separator)
        number_of_columns = len(header_list)
        for column_name in header_list:
            tsv_dict[column_name] = []
        for line in tsv_fd:
            tmp_line = line.strip().split(separator)
            for i in range(0, number_of_columns):
                tsv_dict[header_list[i]].append(tmp_line[i])
    return tsv_dict

def tsv_extract_by_column_value(tsv_file, column_number, column_value, separator="\t",
                                header=False, outfile_prefix=None):
    # column number should start from 0
    # column_value should be string or list of strings

    splited_name = tsv_file.split(".")
    extension = splited_name[-1] if len(splited_name) > 1 else ""
    out_prefix = outfile_prefix if outfile_prefix is not None \
        else ".".join(splited_name[:-1]) if len(splited_name) > 1 else splited_name[0]
    out_fd = open("%s.%s" % (out_prefix, extension), "w")

    column_value_list = column_value if isinstance(column_value, Iterable) else []
    with open(tsv_file, "r") as in_fd:
        if header:
            out_fd.write(in_fd.readline())
        for line in in_fd:
            line_str = line.strip().split(separator)
            if line_str[column_number] in column_value_list:
                out_fd.write(line)
    out_fd.close()



__author__ = 'mahajrod'

from RouToolPa.Tools.Bedtools.MaskFasta import MaskFasta
from RouToolPa.Tools.Bedtools.Flank import Flank
from RouToolPa.Tools.Bedtools.GetFasta import GetFasta
from RouToolPa.Tools.Bedtools.GenomeCov import GenomeCov
from RouToolPa.Tools.Bedtools.Intersect import Intersect
from RouToolPa.Tools.Bedtools.BamToFastq import BamToFastq


Flank = Flank()
GetFasta = GetFasta()
MaskFasta = MaskFasta()
GenomeCov = GenomeCov()
Intersect = Intersect()
BamToFastq = BamToFastq()



#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'

import os
from RouToolPa.Tools.Abstract import Tool


class Clumpify(Tool):
    def __init__(self, path="", max_threads=1):
        Tool.__init__(self, "clumpify.sh", path=path, max_threads=max_threads)

    def remove_optical_duplicates(self, forward_reads, output_prefix, reverse_reads=None, gzip_output=False,
                                  memory_limit="2g"):

        if reverse_reads:
            forward_output = "%s_1.fastq" % output_prefix + (".gz" if gzip_output else "")
            reverse_output = "%s_2.fastq" % output_prefix + (".gz" if gzip_output else "")
        else:
            output = "%s.fastq" % output_prefix + (".gz" if gzip_output else "")

        options = " in1=%s in2=%s" % (forward_reads, reverse_reads) if reverse_reads else " in=%s" % forward_reads
        options += " out1=%s out2=%s" % (forward_output, reverse_output) if reverse_reads else " out=%s" % output

        options += " dedupe"
        options += " optical"
        options += " -Xmx%s" % str(memory_limit)
        self.execute(options=options)


if __name__ == "__main__":
    pass


